#!/bin/csh
#
#
#   Do the backprojection based on the input Spider stack file, angle file, and select file. Several iterations incluing 3D mask, 2D mask, and low-pass filter are applied.
#
#
#   "usage: SPRET-backprojection_forST3DR.com [input_stack_no_ext] [output_prefix] [angle_file_noext] [select_file_noext] [apix] [mask_no_ext] [protein_molecular_weight(KD)]
#
#      where:
#
#          [input_stack_no_ext]: the input Spider stack file to do backprojection.
#     
#          [output-prefix]:      the prefix of the output 3D volumes.
#
#          [angle_file_noext]:   the file contains Euler Angles for the backprojection, could be created by SPRET-create_angle.com, and without extension .dat.
#
#          [select_file_noext]:  the file contains particle selection for the backprojection, could be created by SPRET-create_select.com, and without extension .dat.
#
#          [apix]:               a/pix.
#
#          [mask_no_ext]:        the mask file same as the one which is applied to input stack, for the purpose of making same mask as the input ones in the *mixture.dat and in the refinement processes.
#
#
#	Lei Zhang @ REN Lab, UCSF, Jul. 15 PST 2009
#
#   Specialized for the request of SPRET-ST3DR.com
#
#---------------------------------------------------------------------------------------------------------------------------------------
#
if ($#argv != 9) then
  echo "Do the backprojection based on the input Spider stack file, angle file, and select file."
  echo "usage: SPRET-backprojection_forST3DR_IP.com [input_stack_no_ext] [output_prefix] [angle_file_noext] [select_file_noext] [apix] [mask_no_ext] [protein_molecular_weight(KD)] [IP(2D)] [IP(3D)]"
  echo "such as: SPRET-backprojection_forST3DR_IP.com par0001msknm Tomo01-par0001 myang mysel 2.812 ../mask_fq 208 6 2"
  exit
endif
#
#
rm -r /tmp/eman2*
rm -r ~/.eman2
#
alias MATH 'set \!:1 = `echo "\!:3-$" | bc -l`'
#
set input=$1
set output=$2
#set size=$2
set angfile=$3
set selectfile=$4
set apix=$5
set maskfile=$6
set KD=$7
set folder_exist = `ls -dir refivolzz* | wc -l`
@ folderNum = ${folder_exist} + 1
set i3=`printf "%03d" ${folderNum}`
set totNum1 = `more ${angfile}.dat | wc -l`
@ totalNum = ${totNum1} - 1
set refifolder = "refivolzzz"${i3}
set inputlp=${input}lp
set IP2D=$8
set IP3D=$9
MATH apix2D = ${apix} / ${IP2D}
MATH apix3D = ${apix} / ${IP3D}
#
SPRET-create_select_odd.com ${selectfile} ${selectfile}o
SPRET-create_select_even.com ${selectfile} ${selectfile}e
SPRET-create_select.com zz-sel-all 1-${totalNum}
#
#
spider dat << eot
;
;Read parameters from input strings.
;
[apix]=${apix}
;
[KD]=${KD}
;

;
[one]=1
[zero]=0
;
fi [size]
  ${input}@0000001
  2
;
[IP2D]=${IP2D}
[IP3D]=${IP3D}
[IP2Dsize]=[IP2D]*[size]
[IP3Dsize]=[IP3D]*[size]
;
[first]=1
;
ud n,[nums]
  ${selectfile}
;
ud n,[last]
  ${angfile}
;
ud s,[last],[ztemp],[maxang]
  ${angfile}
;
[maxang]=abs([maxang])
;
[hsize]=int([size]/2)
;
[lp_max]=[size]/6*[apix]
;
[five_pix]=5*[apix]
;
[ffqv10]=[apix]*[size]/10
[lpdivd4]=[apix]*[size]/4
;
; Create blank file.
;
mo
  blank
  [size],[size]
  b
  0.0
;
vm
SPRET-create_unselect.com ${selectfile} zzmyunselect {****[last]}
;
; At first, completely back projection of the raw data without any tricks.
;
vm
SPRET-spistack_mask.com ${input} ${input}msk ${maskfile}
bp 3f
  ${input}msk@*******
  ${selectfile}
  ${angfile}
  *
  ${output}
;
bp 3f
  ${input}msk@*******
  ${selectfile}o
  ${angfile}
  *
  ${output}tempo
;
bp 3f
  ${input}msk@*******
  ${selectfile}e
  ${angfile}
  *
  ${output}tempe
;
[fsc_count]=0
;
rf 3 [fsc_pix] [fsc_frq]
  ${output}tempo
  ${output}tempe
  (1)
  (0.6,1.2)
  W
  [maxang]
  (3.0)
  ${output}-rf3doc
;
;
[resolution]=int(1/[fsc_frq]*[apix])
[resolution2]=[resolution]*2
;
vm
proc3d ${output}.dat ${output}.mrc apix=${apix} mrc
;
vm
proc3d ${output}.dat ${output}-lp008.mrc apix=${apix} lp=8
;
vm
proc3d ${output}.dat ${output}-lp{***[resolution]}.mrc apix=${apix} lp={****[resolution]}
;
vm
proc3d ${output}.dat ${output}-lp{***[resolution2]}.mrc apix=${apix} lp={****[resolution2]}
;
;
de 
  ${output}-record-fsc
de
  ${output}-record-shift-statistics
de
  ${output}-record-shift-change1
de
  ${output}-record-shift-change2
;
;
[real_resolution]=1/[fsc_frq]*[apix]
;
sd [one],[fsc_pix],[fsc_frq],[real_resolution]
  ${output}-record-fsc
;

[lpCC]=1/[fsc_frq]*[apix]*1.5
[lpCCcut]=[lpCC]*2/3
[lpnormal]=1/[fsc_frq]*[apix]*1.5
;
;
vm
rm -r ${refifolder}
vm
mkdir ${refifolder}
;
vm
cp ${output}-rf3doc.dat ${refifolder}/rf3doc000.dat
vm
cp ${input}.dat ${refifolder}/2DRefi000.dat
vm
cp ${input}.dat ${refifolder}/2Ds000.dat
;
cp
${output}
${refifolder}/3DRefi000
;
  pj 3q
    ${refifolder}/3DRefi000
    [hsize]
    [first]-[last]
    ${angfile}
    ${refifolder}/prjztemp@*******

cp
${output}
${output}-loop0
;
cp
${output}
${refifolder}/3Ds000
;
do lb51 [x1]=1,[nums]
  ud s,[x1],[xxx]
    ${selectfile}
  sd [xxx],[zero],[zero]
    ${refifolder}/shifts-total000
lb51
  sd e
${refifolder}/shifts-total000
;
cp
${refifolder}/2Ds000@
${output}-loop0-2Ds@

;
cp
${refifolder}/prjztemp@
${output}-loop0-prj@
;
;
vm
proc3d ${output}-loop0.dat ${output}-loop0-lp8.mrc apix=${apix} lp=8

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;First round, no 3D special mask, no 2D special mask. Begin
;
;
[Itotal]=0
[volume_last]=-1
[volume_next]=0
;
do lb01 [Ilb01]=1,1
;
  [volume_last]=[volume_last]+1
  [volume_next]=[volume_next]+1
;
  pj 3q
    ${refifolder}/3DRefi{***[volume_last]}
    [hsize]
    [first]-[last]
    ${angfile}
    ${refifolder}/prjtemp@*******
;
;    vm
;SPRET-spistack_add_blank.com ${refifolder}/prjtemp zzmyunselect blank
    vm
SPRET-spistack_add_blank.com ${refifolder}/2DRefi000 zzmyunselect blank
;
;  [I_lp_num]=int([lpCC]/[apix])
;
  [factor]=0.5
;
  vm
cp ${refifolder}/2DRefi000.dat ${refifolder}/2DRefi001.dat
;
;  [lptimes]=10  To test the lptimes from 10 to 20 09142009
  [lptimes]=20
;
  [lp_count]=0
;
  [if_lpstop]=0
;
  do lb02 [Ilb02]=0,[lptimes]
;  do lb02 [Ilb02]=0,0
;
    [lp_count]=[lp_count]+1
    if([if_lpstop].eq.1) goto LB92
;
    if([Ilb02].ge.2) then
;vm
;cp ${output}-record-fsc.dat ${output}-record-fsctemp.dat
;    ud s,[Itotal],[fsc_pix_0],[fsc_frq_0],[real_resolution_0]
;      ${output}-record-fsctemp
    [Itotal_f1]=[Itotal]
    endif
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
    [ifstable]=0
;
    [stable_count]=0
;
    do lb03 [Ilb03]=0,60
;
      [stable_count]=[stable_count]+1
;
      if([ifstable].gt.0) goto LB91
;
      [Itotal_last]=[Itotal]
      [Itotal]=[Itotal]+1
;
      [lp1fre]=[apix]/[lpCC]+(0.5-[apix]/[lpCC])/[lptimes]*[Ilb02]
;      [lp2fre]=[apix]/[lpCC]+(0.5-[apix]/[lpCC])/[lptimes]*([Ilb02]+1)  This line is blocked as to test the next line. 09142009.
      [lp2fre]=[apix]/[lpCC]+(0.5-[apix]/[lpCC])/[lptimes]*([Ilb02]+1+0.5)
;
;      [lp1fre]=[apix]/[lpCC]
;      [lp2fre]=[apix]/[lpCC]*2
;
      if([lp1fre].ge.0.5) [lp1fre]=0.5
      if([lp2fre].ge.0.5) [lp2fre]=0.5
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;09142009 To test 09142009B Section instead of the following 7 lines.
;      [lp1]=[apix]/[lp1fre]*10
;      [lpmax]=[lp_max]*10
;      if([lp1].ge.[lpmax]) [lp1]=[lpmax]
;      [lp2]=[apix]/[lp2fre]*10
;      if([lp2].ge.[lpmax]) [lp2]=[lpmax]*2/3
;      [lpzp]=[lp1]*2/3
;      if([lp2].lt.[lpzp]) [lp2]=[lpzp]

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;09142009

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;09142009B
;
      [lp1]=[apix]/[lp1fre]*10
      [lpmax]=[lp_max]*10
      if([lp1].ge.[lpmax]) [lp1]=[lpmax]
      [lp2]=[apix]/[lp2fre]*10
      if([lp2].ge.[lpmax]) [lp2]=[lpmax]*2/3
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;09142009B


;
      sd [Itotal],[lp1],[lp2],[volume_next],[lp_count],[stable_count]
        zz-record-lp
;
;      vm
;SPRET-spistack_add_blank.com ${refifolder}/prjtemp zzmyunselect blank
      vm
SPRET-spistack_add_blank.com ${refifolder}/2DRefi001 zzmyunselect blank
      vm
SPRET-spistack_norm.com ${refifolder}/prjtemp ${refifolder}/prjtempnm
      vm
SPRET-spistack_lpg_forST3DR.com ${refifolder}/prjtempnm ${refifolder}/prjtemplp ${apix} {******[lp1]} {******[lp2]}
      vm
SPRET-spistack_lpg_forST3DR.com ${refifolder}/2DRefi001 ${refifolder}/2DRefi001lp ${apix} {*******[lp1]} {*******[lp2]}
;
; apply mask
;
      do lb52 [xlb12]=1,[nums]
        ud ic,[xlb12],[x20]
          ${selectfile}
; 
        mu
          ${refifolder}/prjtemplp@{*******[x20]}
          ${maskfile}
          ${refifolder}/prjlpmsk@{*******[x20]}
          *
        mu
          ${refifolder}/2DRefi001lp@{*******[x20]}
          ${maskfile}
          ${refifolder}/2DRefi001lpmsk@{*******[x20]}
          *
;
      lb52
;
;
      ud ice
        ${selectfile}
;
; calculate shifts
;
      do lb53 [x21]=1,[nums]
        ud ic,[x21],[x20]
          ${selectfile}
;
        CC N
          ${refifolder}/2DRefi001lpmsk@{*******[x20]}
          ${refifolder}/prjlpmsk@{*******[x20]}
          _5
;
        pk [x11],[x12]
          _5
          (1)
        sd [x20],[x11],[x12]
          ${refifolder}/shifts{***[Itotal]}
;
        ud ic,[x21],[x20]
          ${selectfile}
;
        ud s,[x20],[x111],[x121]
          ${refifolder}/shifts-total{***[Itotal_last]}
;
        [x112]=[x111]+[x11]
        [x122]=[x121]+[x12]
;
; To check the shifts to make sure them less than [size]/3
;
        [absx112]=abs([x112])
        [absx122]=abs([x122])
;
        [tta]=[size]/3
        if([absx112].gt.[tta]) [x112]=0
        if([absx122].gt.[tta]) [x122]=0
;
        sd [x20],[x112],[x122]
          ${refifolder}/shifts-total{***[Itotal]}
;
;        SH F
;          ${refifolder}/2DRefi000@{*******[x20]}
;          ${refifolder}/2DRefi001@{*******[x20]}
;          -[x112],-[x122]
;       ud s,[x20],[x11],[x12]
;         ${refifolder}/shifts{***[Itotal]}
        SH F
          ${refifolder}/2DRefi000@{*******[x20]}
          ${refifolder}/2DRefi001@{*******[x20]}
          -[x112],-[x122]

;
      lb53
;
cp
${refifolder}/2DRefi001@
${refifolder}/2Ds{***[Itotal]}@
      ud ice
        ${selectfile}
    
;
      sd e
${refifolder}/shifts{***[Itotal]}
      sd e
${refifolder}/shifts-total{***[Itotal]}
;
; Apply mask to data before them backprojection.
;
vm
SPRET-spistack_add_blank.com ${refifolder}/2DRefi001 zzmyunselect blank
;
vm
SPRET-spistack_mask.com ${refifolder}/2DRefi001 ${refifolder}/2DRefi001msk ${maskfile}
;
      BP 3f
        ${refifolder}/2DRefi001msk@*******
        ${selectfile}
        ${angfile}
        *
        ${refifolder}/3DRefi001temp
;
      BP 3f
        ${refifolder}/2DRefi001msk@*******
        ${selectfile}o
        ${angfile}
        *
        ${refifolder}/tmpvol1
;
      BP 3f
        ${refifolder}/2DRefi001msk@*******
        ${selectfile}e
        ${angfile}
        *
        ${refifolder}/tmpvol2
;
[volume_end]=3*[KD]
;
[volume_end_voxels3]=[volume_end]*1000/0.81/[apix]/[apix]/[apix]
[volume_end_voxels3a]=[volume_end_voxels3]*4/5
;
[equal_protein_size]=(2*[KD]*1000/0.81)**(1/3)
[size_diff]=[size]-[equal_protein_size]
;[expansion_times]=int([size_diff]/8)
[expansion_times]=int([size]/2/8)
[tri_equal_protein_size]=[equal_protein_size]/3/[apix]
[tri_box_size]=[size]/3
;
; lp_min is in the unit of A, not voxels.
;
[lp_min]=[equal_protein_size]/2
[lp_thr]=[lpCC]
if([lp_thr].lt.[lp_min]) [lp_thr]=[lp_min]
if([lp_thr].gt.[lp_max]) [lp_thr]=[lp_max]
;
;[fq_min]=[size]/2*[apix]
[fq_min]=[equal_protein_size]
vm
proc3d ${refifolder}/3DRefi001temp.dat ${refifolder}/3DRefi001templp.mrc apix=${apix} lp={*****[lp_thr]}
cp from ccp4
${refifolder}/3DRefi001templp.mrc
${refifolder}/3DRefi001templp
n
;
fv [thr_cen]
${refifolder}/3DRefi001templp
[volume_end_voxels3a]
;
ar
${refifolder}/3DRefi001templp
${refifolder}/3DRefi001templp2
P1/[thr_cen]
;
[equal_protein_size]=(2*[KD]*1000/0.81)**(1/3)
[tri_equal_protein_size]=[equal_protein_size]/3/[apix]
;
vm
proc3d ${refifolder}/3DRefi001templp2.dat ${refifolder}/3DRefi001templp2.mrc apix=${apix} automask2={****[tri_box_size]},1,10
;
cp from ccp4
mask.mrc
mask
n
;
mu
${refifolder}/3DRefi001templp
mask
${refifolder}/3DRefi001templpmask
*
;
fs [zzd1],[zzd2],[zzd3],[maskSTD]
mask
;
if([maskSTD].eq.0) then
[cx]=0
[cy]=0
[cz]=0
endif
;
if([maskSTD].gt.0) then
cg 3 [cx],[cy],[cz]
${refifolder}/3DRefi001templpmask
[thr_cen]
endif
;
sh f
${refifolder}/3DRefi001temp
${refifolder}/3DRefi001
-[cx],-[cy],-[cz]
;
cp
${refifolder}/3DRefi001
${refifolder}/3Ds{***[Itotal]}
;
      pj 3q
        ${refifolder}/3DRefi001
        [hsize]
        [first]-[last]
        ${angfile}
        ${refifolder}/prjtemp@*******
;
      rf 3 [fsc_pix] [fsc_frq]
        ${refifolder}/tmpvol1
        ${refifolder}/tmpvol2
        (1)
        (0.6,1.2)
        W
        [maxang]
        (3.0)
        ${output}-rf3doc
;
;
      rf 3 [fsc_pix] [fsc_frq]
        ${refifolder}/tmpvol1
        ${refifolder}/tmpvol2
        (1)
        (0.6,1.2)
        W
        [maxang]
        (3.0)
        ${refifolder}/rf3doc{***[Itotal]}
;
;
      [lpCC]=1/[fsc_frq]*[apix]*1.5
      [lpCCcut]=[lpCC]/2
      [real_resolution]=1/[fsc_frq]*[apix]
;
;
      sd [Itotal],[fsc_pix],[fsc_frq],[real_resolution],[volume_next],[lp_count],[stable_count],[lp1],[lp2]
        ${output}-record-fsc
;
      vm
SPRET-doc_shift_statistics_refi.com 1 zz-templast-2 ${refifolder}/shifts-total{***[Itotal]}
;
      ud s,[one],[absPerSet],[absPerImg]
        zz-templast-2
;
      sd [Itotal],[absPerSet],[absPerImg],[volume_next],[lp_count],[stable_count]
        ${output}-record-shift-statistics
;
; To check the creteria: shift changes
;
        [last_shift4]=[Itotal]-0
        [last_shift3]=[Itotal]-1
        [last_shift2]=[Itotal]-2
        [last_shift1]=[Itotal]-3
;
        if([last_shift4].lt.0) [last_shift4]=0
        if([last_shift3].lt.0) [last_shift3]=0
        if([last_shift2].lt.0) [last_shift2]=0
        if([last_shift1].lt.0) [last_shift1]=0
vm
SPRET-doc_shift_statistics_refi.com 4 zz-temp-shift_sta1 ${refifolder}/shifts-total{***[last_shift1]} ${refifolder}/shifts-total{***[last_shift2]} ${refifolder}/shifts-total{***[last_shift3]} ${refifolder}/shifts-total{***[last_shift4]}
;
        vm
SPRET-doc_shift_statistics_refi.com 1 zz-temp-shift_sta2 ${refifolder}/shifts-total{***[last_shift4]}
        vm
SPRET-doc_shift_statistics_refi.com 1 zz-temp-shift_sta3 ${refifolder}/shifts{***[last_shift4]}
;
vm
cp zz-temp-shift_sta1.dat zz-temp-shift_sta11.dat
vm
cp zz-temp-shift_sta2.dat zz-temp-shift_sta21.dat
vm
cp zz-temp-shift_sta3.dat zz-temp-shift_sta31.dat
;
if([stable_count].ge.4) then
        ud s,[one],[absPerSet1],[absPerImg1]
          zz-temp-shift_sta11
        ud s,[one],[absPerSet2],[absPerImg2]
          zz-temp-shift_sta21
        ud s,[one],[absPerSet3],[absPerImg3]
          zz-temp-shift_sta31
;
if([absPerImg2].gt.0) [absChange]=abs(([absPerImg1]-[absPerImg2])/[absPerImg2])
if([absPerImg2].eq.0) [absChange]=0
;
sd [Itotal],[absPerSet1],[absPerImg1],[absPerSet2],[absPerImg2],[absChange]
${output}-record-shift-change1
sd [Itotal],[absPerImg3],[volume_next],[lp_count],[stable_count]
${output}-record-shift-change2
; 
        [judge1]=0
        if([absPerImg3].lt.0.01) [judge1]=1
        [judge2]=0
        if([absChange].le.0.01) [judge2]=1
        [judge3]=[judge1]*[judge2]
        if([judge3].eq.1) [ifstable]=1
;
;        if([absPerImg3].lt.0.05) then
;           if([absChange].le.0.01) then
;              [ifstable]=1
;           endif
;        endif
endif
;
LB91
;
    lb03
; 
    [abs_lpstop]=1
if([Ilb02].ge.2) then
    [Itotal_f2]=[Itotal]
;vm
;cp ${output}-record-fsc.dat ${output}-record-fsctemp.dat
;    ud s,[Itotal],[fp1],[fq1],[rr1]
;      ${output}-record-fsctemp
vm
cp ${refifolder}/shifts-total{***[Itotal_f1]}.dat zz1015a.dat
vm
cp ${refifolder}/shifts-total{***[Itotal_f2]}.dat zz1015b.dat
vm
SPRET-doc_shift_statistics_diff.com zz1015c zz1015a zz1015b
vm
cp zz1015c.dat zz1015d.dat
ud s,[one],[abs_lpstop]
zz1015d
endif
if([abs_lpstop].le.0.01) [if_lpstop]=1
LB92
  lb02
;
lb01
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;First round, no 3D mask, no 2D mask. End
;
;Assuming the protein has a density of 0.81 Da/A^3.
;
[total_volume]=4/3*3.14159/8*[size]*[size]*[size]*[apix]*[apix]*[apix]
;
;Total molecular weight of the box in the unit of KDa.
;
[total_MW]=0.81*[total_volume]/1000
;
[volume_ratio]=[total_MW]/[KD]
;
;Volume mask begin from 1/3*box volume to 3*protein volume.
;
[volume_begin]=1/2*[total_MW]
;
[volume_end]=3*[KD]
;
[volume_end_voxels3]=[volume_end]*1000/0.81/[apix]/[apix]/[apix]
;
;[volume_end1]=3.5*[KD]
;[volume_end2]=4*[KD]
;
[volume_decrease_nums]=6
;
[volume_decrease_step]=(1/[volume_end]-1/[volume_begin])/5
;
;Equal protein size = Diamerter of a sphere of volume V.
;
[equal_protein_size]=(2*[KD]*1000/0.81)**(1/3)
[tri_equal_protein_size]=[equal_protein_size]/3/[apix]
;
; lp_min is in the unit of A, not voxels.
;
[lp_min]=[equal_protein_size]/2
;
cp
${refifolder}/3DRefi001
${refifolder}/3DRefiMask000
;
cp
${refifolder}/prjtemp@
${output}-loop1-prj@
;
cp
${refifolder}/2DRefi001@
${output}-loop1-2Ds@
;
cp
${refifolder}/3DRefi001
${output}-loop1
vm
proc3d ${output}-loop1.dat ${output}-loop1-lp8.mrc apix=${apix} lp=8
;
vm
cp ${refifolder}/2DRefi001.dat ${refifolder}/2DRefiMask000.dat

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;Second round, 3D mask, 2D mask. Begin
;
;
[volume_last]=-1
[volume_next]=0
;
[Ilb11end]=2+[volume_decrease_nums]-1
;
do lb11 [Ilb11]=2,[Ilb11end]
;
  [volume_last]=[volume_last]+1
  [volume_next]=[volume_next]+1
;
  [volume_decreat_index]=[Ilb11]-2
  [volume_desired]=1/(1/[volume_begin]+[volume_decrease_step]*[volume_decreat_index])
;
  [volume_desired00_voxels3]=[volume_desired]*1000/0.81/[apix]/[apix]/[apix]
;
  [volume_desired01_voxels3]=[volume_desired]*1000/0.81/[apix]/[apix]/[apix]+[volume_desired00_voxels3]*1/15
;
  [volume_desired02_voxels3]=[volume_desired]*1000/0.81/[apix]/[apix]/[apix]+[volume_desired00_voxels3]*2/15
;
  [volume_desired03_voxels3]=[volume_desired]*1000/0.81/[apix]/[apix]/[apix]+[volume_desired00_voxels3]*3/15
;
  [volume_desired04_voxels3]=[volume_desired]*1000/0.81/[apix]/[apix]/[apix]+[volume_desired00_voxels3]*4/15
;
  [volume_desired05_voxels3]=[volume_desired]*1000/0.81/[apix]/[apix]/[apix]+[volume_desired00_voxels3]*5/15
;
  [lp_thr]=[lpCC]
  if([lp_thr].lt.[lp_min]) [lp_thr]=[lp_min]
  if([lp_thr].gt.[lp_max]) [lp_thr]=[lp_max]
;
  vm
  proc3d ${refifolder}/3DRefiMask{***[volume_last]}.dat ${refifolder}/3DRefiMask{***[volume_last]}-lp.mrc apix=${apix} lp={****[lp_thr]}
;
  cp from ccp4
${refifolder}/3DRefiMask{***[volume_last]}-lp.mrc
${refifolder}/3DRefiMask{***[volume_last]}-lp
n
;
;  fv [thr_end]
;    ${refifolder}/3DRefiMask{***[volume_last]}-lp
;    [volume_end_voxels3]


[volume_end_voxels3a]=[volume_end_voxels3]*4/5

  fv [thr_end]
    ${refifolder}/3DRefiMask{***[volume_last]}-lp
    [volume_end_voxels3a]

;
; usage: SPRET-3D_mask_expansion.com [input_mrc_3D] [output_mrc_3D] [Apix] [center_radius] [threshold] [iteration_times]
[thr_end_1000]=[thr_end]*1000
;[thr_end_1000]=[thr_end]*1000*4/5
  vm
SPRET-3D_mask_expansion.com ${refifolder}/3DRefiMask{***[volume_last]}-lp.mrc ${refifolder}/3DRefiMask{***[volume_last]}-lpmask.mrc ${apix} {*****[tri_box_size]} {*****[thr_end_1000]} {*****[expansion_times]}
;
  cp from ccp4
    ${refifolder}/3DRefiMask{***[volume_last]}-lpmask.mrc
    ${refifolder}/3DRefiMask{***[volume_last]}-lpmask
    n
;
  fv [threshold_desired00]
    ${refifolder}/3DRefiMask{***[volume_last]}-lpmask
    [volume_desired00_voxels3]
;
  fv [threshold_desired01]
    ${refifolder}/3DRefiMask{***[volume_last]}-lpmask
    [volume_desired01_voxels3]
;
  fv [threshold_desired02]
    ${refifolder}/3DRefiMask{***[volume_last]}-lpmask
    [volume_desired02_voxels3]
;
  fv [threshold_desired03]
    ${refifolder}/3DRefiMask{***[volume_last]}-lpmask
    [volume_desired03_voxels3]
;
  fv [threshold_desired04]
    ${refifolder}/3DRefiMask{***[volume_last]}-lpmask
    [volume_desired04_voxels3]
;
  fv [threshold_desired05]
    ${refifolder}/3DRefiMask{***[volume_last]}-lpmask
    [volume_desired05_voxels3]
;
  ar
    ${refifolder}/3DRefiMask{***[volume_last]}-lpmask
    ${refifolder}/3DRefiMask-lp-temp00
    P1/[threshold_desired00]
;
  th m
    ${refifolder}/3DRefiMask-lp-temp00
    ${refifolder}/3DRefiMask-lp-temp00thm
    B
    (1.0)
;
  ar
    ${refifolder}/3DRefiMask{***[volume_last]}-lpmask
    ${refifolder}/3DRefiMask-lp-temp01
    P1/[threshold_desired01]
;
  th m
    ${refifolder}/3DRefiMask-lp-temp01
    ${refifolder}/3DRefiMask-lp-temp01thm
    B
    (1.0)
;
  ar
    ${refifolder}/3DRefiMask{***[volume_last]}-lpmask
    ${refifolder}/3DRefiMask-lp-temp02
    P1/[threshold_desired02]
;
  th m
    ${refifolder}/3DRefiMask-lp-temp02
    ${refifolder}/3DRefiMask-lp-temp02thm
    B
    (1.0)
;
  ar
    ${refifolder}/3DRefiMask{***[volume_last]}-lpmask
    ${refifolder}/3DRefiMask-lp-temp03
    P1/[threshold_desired03]
;
  th m
    ${refifolder}/3DRefiMask-lp-temp03
    ${refifolder}/3DRefiMask-lp-temp03thm
    B
    (1.0)
;
  ar
    ${refifolder}/3DRefiMask{***[volume_last]}-lpmask
    ${refifolder}/3DRefiMask-lp-temp04
    P1/[threshold_desired04]
;
  th m
    ${refifolder}/3DRefiMask-lp-temp04
    ${refifolder}/3DRefiMask-lp-temp04thm
    B
    (1.0)
;
  ar
    ${refifolder}/3DRefiMask{***[volume_last]}-lpmask
    ${refifolder}/3DRefiMask-lp-temp05
    P1/[threshold_desired05]
;
  th m
    ${refifolder}/3DRefiMask-lp-temp05
    ${refifolder}/3DRefiMask-lp-temp05thm
    B
    (1.0)
;
  ad
    ${refifolder}/3DRefiMask-lp-temp00thm
    ${refifolder}/3DRefiMask-lp-temp01thm
    ${refifolder}/3DRefiMask-ad{***[volume_next]}
    ${refifolder}/3DRefiMask-lp-temp02thm
    ${refifolder}/3DRefiMask-lp-temp03thm
    ${refifolder}/3DRefiMask-lp-temp04thm
    ${refifolder}/3DRefiMask-lp-temp05thm
    *
; 
  ar
    ${refifolder}/3DRefiMask-ad{***[volume_next]}
    ${refifolder}/3DRefiMask-adar{***[volume_next]}    
    P1/6
;
  vm
    SPRET-3D_fq.com ${refifolder}/3DRefiMask-adar{***[volume_next]} ${refifolder}/3DMaskz{***[volume_next]} ${apix} {****[fq_min]}

ar
${refifolder}/3DMaskz{***[volume_next]}
${refifolder}/3DMaskzz{***[volume_next]}
P1/0.5
;
th
${refifolder}/3DMaskzz{***[volume_next]}
${refifolder}/3DMaskzzz{***[volume_next]}
A
(1.0)
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;Revised 20101018 begin - adding a ar and th process
;  vm
;    SPRET-3D_fq.com ${refifolder}/3DMaskzzz{***[volume_next]} ${refifolder}/3DMask{***[volume_next]} ${apix} {****[five_pix]}
  vm
    SPRET-3D_fq.com ${refifolder}/3DMaskzzz{***[volume_next]} ${refifolder}/3DMaskzb{***[volume_next]} ${apix} {****[five_pix]}

ar
${refifolder}/3DMaskzb{***[volume_next]}
${refifolder}/3DMaskzzb{***[volume_next]}
P1/0.5
;
th
${refifolder}/3DMaskzzb{***[volume_next]}
${refifolder}/3DMask{***[volume_next]}
A
(1.0)
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;Revised 20101018 end - adding a ar and th process
  pj 3q
    ${refifolder}/3DRefiMask-lp-temp05thm
    [hsize]
    [first]-[last]
    ${angfile}
    ${refifolder}/prjtemp2Dmask00@*******
;
  vm
    SPRET-spistack_thm.com ${refifolder}/prjtemp2Dmask00 ${refifolder}/prjtemp2Dmask00thm 1
  vm
    SPRET-spistack_fq.com ${refifolder}/prjtemp2Dmask00thm ${refifolder}/2Dmaskz{***[volume_next]} ${apix} {****[fq_min]}    
ar
${refifolder}/2Dmaskz{***[volume_next]}@
${refifolder}/2Dmaskzz{***[volume_next]}@
P1/0.5
;
  vm
    SPRET-spistack_th.com ${refifolder}/2Dmaskzz{***[volume_next]} ${refifolder}/2Dmaskzzz{***[volume_next]} 1
  vm
    SPRET-spistack_fq.com ${refifolder}/2Dmaskzzz{***[volume_next]} ${refifolder}/2Dmask{***[volume_next]} ${apix} {****[five_pix]}
;
  mu
    ${refifolder}/3DRefiMask{***[volume_last]}
    ${refifolder}/3DMask{***[volume_next]}
    ${refifolder}/3DRefiMasked{***[volume_next]}
    *
;
  pj 3q
    ${refifolder}/3DRefiMasked{***[volume_next]}
    [hsize]
    [first]-[last]
    ${angfile}
    ${refifolder}/prjtemp@*******
;
;    vm
;SPRET-spistack_add_blank.com ${refifolder}/prjtemp zzmyunselect blank
    vm
SPRET-spistack_add_blank.com ${refifolder}/2DRefi000 zzmyunselect blank
;
;  [I_lp_num]=int([lpCC]/[apix])
;
  [factor]=0.5
;
;
  [lp_count]=0
;
  [if_lpstop]=0
;
  do lb12 [Ilb12]=0,[lptimes]
;
    [lp_count]=[lp_count]+1
;
    if([if_lpstop].eq.1) goto LB82
;
    if([Ilb12].ge.2) then
;vm
;cp ${output}-record-fsc.dat ${output}-record-fsctemp.dat
;    ud s,[Itotal],[fsc_pix_0],[fsc_frq_0],[real_resolution_0]
;      ${output}-record-fsctemp
    [Itotal_f1]=[Itotal]
    endif
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
    [ifstable]=0
;
    [stable_count]=0
;
    [Ilb12c]=[Ilb12]
    do lb13 [Ilb13]=0,60
;
      [stable_count]=[stable_count]+1
;
      if([ifstable].gt.0) goto LB81
;
      [Itotal_last]=[Itotal]
      [Itotal]=[Itotal]+1
;
      [lp1fre]=[apix]/[lpCC]+(0.5-[apix]/[lpCC])/[lptimes]*[Ilb12]
;      [lp2fre]=[apix]/[lpCC]+(0.5-[apix]/[lpCC])/[lptimes]*([Ilb12]+1)  This line is blocked as to test the next line. 09142009.
      [lp2fre]=[apix]/[lpCC]+(0.5-[apix]/[lpCC])/[lptimes]*([Ilb12]+1+0.5)

;      [lp1fre]=[apix]/[lpCC]
;      [lp2fre]=[apix]/[lpCC]*2
;
      if([lp1fre].ge.0.5) [lp1fre]=0.5
      if([lp2fre].ge.0.5) [lp2fre]=0.5
;
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;09142009 To test 09142009B Section instead of the following 7 lines.
;      [lp1]=[apix]/[lp1fre]*10
;      [lpmax]=[lp_max]*10
;      if([lp1].ge.[lpmax]) [lp1]=[lpmax]
;      [lp2]=[apix]/[lp2fre]*10
;      if([lp2].ge.[lpmax]) [lp2]=[lpmax]*2/3
;      [lpzp]=[lp1]*2/3
;      if([lp2].lt.[lpzp]) [lp2]=[lpzp]

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;09142009

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;09142009B
;
      [lp1]=[apix]/[lp1fre]*10
      [lpmax]=[lp_max]*10
      if([lp1].ge.[lpmax]) [lp1]=[lpmax]
      [lp2]=[apix]/[lp2fre]*10
      if([lp2].ge.[lpmax]) [lp2]=[lpmax]*2/3
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;09142009B

      sd [Itotal],[lp1],[lp2],[volume_next],[lp_count],[stable_count]
        zz-record-lp
;
;      vm
;SPRET-spistack_add_blank.com ${refifolder}/prjtemp zzmyunselect blank
      vm
SPRET-spistack_add_blank.com ${refifolder}/2DRefi001 zzmyunselect blank
      vm
SPRET-spistack_mu.com ${refifolder}/2DRefi001 ${refifolder}/2Dmask{***[volume_next]} ${refifolder}/2DRefi001mu
      vm
SPRET-spistack_norm.com ${refifolder}/2DRefi001mu ${refifolder}/2DRefi001munm
      vm
SPRET-spistack_norm.com ${refifolder}/prjtemp ${refifolder}/prjtempnm
      vm
SPRET-spistack_lpg_forST3DR.com ${refifolder}/prjtempnm ${refifolder}/prjtemplp ${apix} {******[lp1]} {******[lp2]}
      vm
SPRET-spistack_lpg_forST3DR.com ${refifolder}/2DRefi001munm ${refifolder}/2DRefi001mulp ${apix} {*******[lp1]} {*******[lp2]}
;
; apply mask
;
      do lb52 [xlb12]=1,[nums]
        ud ic,[xlb12],[x20]
          ${selectfile}
; 
        mu
          ${refifolder}/prjtemplp@{*******[x20]}
          ${maskfile}
          ${refifolder}/prjlpmsk@{*******[x20]}
          *
        mu
          ${refifolder}/2DRefi001mulp@{*******[x20]}
          ${maskfile}
          ${refifolder}/2DRefi001mulpmsk@{*******[x20]}
          *
;
      lb52
;
;
      ud ice
        ${selectfile}
;
; calculate shifts
;
      do lb53 [x21]=1,[nums]
        ud ic,[x21],[x20]
          ${selectfile}
;
        CC N
          ${refifolder}/2DRefi001mulpmsk@{*******[x20]}
          ${refifolder}/prjlpmsk@{*******[x20]}
          _5
;
        pk [x11],[x12]
          _5
          (1)
;
        ud s,[x20],[x111],[x121]
          ${refifolder}/shifts-total{***[Itotal_last]}
;
        sd [x20],[x11],[x12]
          ${refifolder}/shifts{***[Itotal]}
;
        [x112]=[x111]+[x11]
        [x122]=[x121]+[x12]
;
; To check the shifts to make sure them less than [size]/3
;
        [absx112]=abs([x112])
        [absx122]=abs([x122])
;
        [tta]=[size]/3
        if([absx112].gt.[tta]) [x112]=0
        if([absx122].gt.[tta]) [x122]=0
;
        sd [x20],[x112],[x122]
          ${refifolder}/shifts-total{***[Itotal]}
;
        SH F
          ${refifolder}/2DRefi000@{*******[x20]}
          ${refifolder}/2DRefi001@{*******[x20]}
          -[x112],-[x122]
;

;
      lb53
cp
${refifolder}/2DRefi001@
${refifolder}/2Ds{***[Itotal]}@
;
      ud ice
        ${selectfile}
    
;
      sd e
${refifolder}/shifts-total{***[Itotal]}
;
      sd e
${refifolder}/shifts{***[Itotal]}
;
;
vm
SPRET-spistack_add_blank.com ${refifolder}/2DRefi001 zzmyunselect blank
;
vm
SPRET-spistack_mask.com ${refifolder}/2DRefi001 ${refifolder}/2DRefi001msk ${maskfile}
;
      BP 3f
        ${refifolder}/2DRefi001msk@*******
        ${selectfile}
        ${angfile}
        *
        ${refifolder}/3DRefiMask{***[volume_next]}temp
;
      BP 3f
        ${refifolder}/2DRefi001msk@*******
        ${selectfile}o
        ${angfile}
        *
        ${refifolder}/tmpvol1
;
      BP 3f
        ${refifolder}/2DRefi001msk@*******
        ${selectfile}e
        ${angfile}
        *
        ${refifolder}/tmpvol2
;
vm
proc3d ${refifolder}/3DRefiMask{***[volume_next]}temp.dat ${refifolder}/3DRefiMask{***[volume_next]}templp.mrc apix=${apix} lp={*****[lp_thr]}
cp from ccp4
${refifolder}/3DRefiMask{***[volume_next]}templp.mrc
${refifolder}/3DRefiMask{***[volume_next]}templp
n
;
fv [thr_cen]
${refifolder}/3DRefiMask{***[volume_next]}templp
[volume_end_voxels3]
;
ar
${refifolder}/3DRefiMask{***[volume_next]}templp
${refifolder}/3DRefiMask{***[volume_next]}templp2
P1/[thr_cen]
;
[equal_protein_size]=(2*[KD]*1000/0.81)**(1/3)
[tri_equal_protein_size]=[equal_protein_size]/3/[apix]
;
vm
proc3d ${refifolder}/3DRefiMask{***[volume_next]}templp2.dat ${refifolder}/3DRefiMask{***[volume_next]}templp2.mrc apix=${apix} automask2={****[tri_box_size]},1,8
;
cp from ccp4
mask.mrc
mask
n
;
mu
${refifolder}/3DRefiMask{***[volume_next]}templp
mask
${refifolder}/3DRefiMask{***[volume_next]}templpmask
*
;
cg 3 [cx],[cy],[cz]
${refifolder}/3DRefiMask{***[volume_next]}templpmask
[thr_cen]

fs [thrmax]
${refifolder}/3DRefiMask{***[volume_next]}templpmask

if([thrmax].lt.[thr_cen]) then
[thrzz]=[thrmax]/2
cg 3 [cx],[cy],[cz]
${refifolder}/3DRefiMask{***[volume_next]}templpmask
[thrzz]
endif
;
sh f
${refifolder}/3DRefiMask{***[volume_next]}temp
${refifolder}/3DRefiMask{***[volume_next]}
-[cx],-[cy],-[cz]
;
cp
${refifolder}/3DRefiMask{***[volume_next]}
${refifolder}/3Ds{***[Itotal]}
;
mu
${refifolder}/3DRefiMask{***[volume_next]}
${refifolder}/3DMask{***[volume_next]}
${refifolder}/3DRefiMasked{***[volume_next]}
*
;
      pj 3q
        ${refifolder}/3DRefiMasked{***[volume_next]}
        [hsize]
        [first]-[last]
        ${angfile}
        ${refifolder}/prjtemp@*******
;
      rf 3 [fsc_pix] [fsc_frq]
        ${refifolder}/tmpvol1
        ${refifolder}/tmpvol2
        (1)
        (0.6,1.2)
        W
        [maxang]
        (3.0)
        ${output}-rf3doc
;
      rf 3 [fsc_pix] [fsc_frq]
        ${refifolder}/tmpvol1
        ${refifolder}/tmpvol2
        (1)
        (0.6,1.2)
        W
        [maxang]
        (3.0)
        ${refifolder}/rf3doc{***[Itotal]}
;
;
      [lpCC]=1/[fsc_frq]*[apix]*1.5
      [lpCCcut]=[lpCC]*2/3
      [real_resolution]=1/[fsc_frq]*[apix]
;

;
;
      sd [Itotal],[fsc_pix],[fsc_frq],[real_resolution],[volume_next],[lp_count],[stable_count],[lp1],[lp2]
        ${output}-record-fsc
;
      vm
SPRET-doc_shift_statistics_refi.com 1 zz-templast-2 ${refifolder}/shifts-total{***[Itotal]}
;
      ud s,[one],[absPerSet],[absPerImg]
        zz-templast-2
;
      sd [Itotal],[absPerSet],[absPerImg],[volume_next],[lp_count],[stable_count]
        ${output}-record-shift-statistics
;
; To check the creteria: shift changes
;
        [last_shift4]=[Itotal]-0
        [last_shift3]=[Itotal]-1
        [last_shift2]=[Itotal]-2
        [last_shift1]=[Itotal]-3
;
        if([last_shift4].lt.0) [last_shift4]=0
        if([last_shift3].lt.0) [last_shift3]=0
        if([last_shift2].lt.0) [last_shift2]=0
        if([last_shift1].lt.0) [last_shift1]=0
vm
SPRET-doc_shift_statistics_refi.com 4 zz-temp-shift_sta1 ${refifolder}/shifts-total{***[last_shift1]} ${refifolder}/shifts-total{***[last_shift2]} ${refifolder}/shifts-total{***[last_shift3]} ${refifolder}/shifts-total{***[last_shift4]}
;
        vm
SPRET-doc_shift_statistics_refi.com 1 zz-temp-shift_sta2 ${refifolder}/shifts-total{***[last_shift4]}
        vm
SPRET-doc_shift_statistics_refi.com 1 zz-temp-shift_sta3 ${refifolder}/shifts{***[last_shift4]}
;
vm
cp zz-temp-shift_sta1.dat zz-temp-shift_sta11.dat
vm
cp zz-temp-shift_sta2.dat zz-temp-shift_sta21.dat
vm
cp zz-temp-shift_sta3.dat zz-temp-shift_sta31.dat
;
if([stable_count].ge.4) then
        ud s,[one],[absPerSet1],[absPerImg1]
          zz-temp-shift_sta11
        ud s,[one],[absPerSet2],[absPerImg2]
          zz-temp-shift_sta21
        ud s,[one],[absPerSet3],[absPerImg3]
          zz-temp-shift_sta31
;
if([absPerImg2].gt.0) [absChange]=abs(([absPerImg1]-[absPerImg2])/[absPerImg2])
if([absPerImg2].eq.0) [absChange]=0
;
sd [Itotal],[absPerSet1],[absPerImg1],[absPerSet2],[absPerImg2],[absChange]
${output}-record-shift-change1
sd [Itotal],[absPerImg3],[volume_next],[lp_count],[stable_count]
${output}-record-shift-change2
; 
        [judge1]=0
        if([absPerImg3].lt.0.01) [judge1]=1
        [judge2]=0
        if([absChange].le.0.01) [judge2]=1
        [judge3]=[judge1]*[judge2]
        if([judge3].eq.1) [ifstable]=1
;
;        if([absPerImg3].lt.0.01) then
;           if([absChange].le.0.01) then
;              [ifstable]=1
;           endif
;        endif
endif
;
LB81
;


    lb13
;
    [abs_lpstop]=1
if([Ilb12].ge.2) then
    [Itotal_f2]=[Itotal]
;vm
;cp ${output}-record-fsc.dat ${output}-record-fsctemp.dat
;    ud s,[Itotal],[fp1],[fq1],[rr1]
;      ${output}-record-fsctemp
vm
cp ${refifolder}/shifts-total{***[Itotal_f1]}.dat zz1015a.dat
vm
cp ${refifolder}/shifts-total{***[Itotal_f2]}.dat zz1015b.dat
vm
SPRET-doc_shift_statistics_diff.com zz1015c zz1015a zz1015b
vm
cp zz1015c.dat zz1015d.dat
ud s,[one],[abs_lpstop]
zz1015d
endif
if([abs_lpstop].le.0.01) [if_lpstop]=1
;
LB82
;
  lb12
;
;  cp
;    ${refifolder}/3DRefiMask001
;    ${refifolder}/3DRefiMask{***[volume_next]}
lb11
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;Second round, 3D mask, 2D mask. End
;
;
cp
${refifolder}/3DRefiMask{***[volume_next]}
${output}-loop2
;
vm
proc3d ${output}-loop2.dat ${output}-loop2-lp8.mrc apix=${apix} lp=8
;
;
cp
${refifolder}/prjtemp@
${output}-loop2-prj@
;
cp
${refifolder}/2DRefi001msk@
${output}-loop2-2Ds@
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; IP and find more accurate shifts Begin
[hsizeBP]=[hsize]*[IP3D]
;
ip t
${maskfile}
${maskfile}IP
[IP2Dsize],[IP2Dsize]
;
ip t
${maskfile}
${maskfile}BP
[IP3Dsize],[IP3Dsize]
;
ip t
blank
blankBP
[IP3Dsize],[IP3Dsize]
;
ip t
blank
blankIP
[IP2Dsize],[IP2Dsize]
;
ip
${refifolder}/3DMask{***[volume_next]}
${refifolder}/3DMask{***[volume_next]}BP
[IP3Dsize],[IP3Dsize],[IP3Dsize]
;
;      vm
;SPRET-spistack_add_blank.com ${refifolder}/prjtemp zzmyunselect blank
      vm
SPRET-spistack_add_blank.com ${refifolder}/2DRefi001 zzmyunselect blank
;
vm
SPRET-spistack_ip.com ${refifolder}/2Dmask{***[volume_next]} ${refifolder}/2Dmask{***[volume_next]}IP {*****[IP2Dsize]} {*****[IP2Dsize]}
vm
SPRET-spistack_ip.com ${refifolder}/prjtemp ${refifolder}/prjtempIP {*****[IP2Dsize]} {*****[IP2Dsize]}
vm
SPRET-spistack_ip.com ${refifolder}/2DRefi000 ${refifolder}/2DRefi000IP {*****[IP2Dsize]} {*****[IP2Dsize]}
vm
SPRET-spistack_ip.com ${refifolder}/2DRefi001 ${refifolder}/2DRefi001IP {*****[IP2Dsize]} {*****[IP2Dsize]}
;
;
;
;
;
;
    [ifstable]=0
;
    [stable_count]=0
;
do lb44 [xlb44]=1,60
;
      [stable_count]=[stable_count]+1
;
      if([ifstable].gt.0) goto LB39
;
[Itotal]=[Itotal]+1
[Itotal_last]=[Itotal]-1
;
      [lp1fre]=[apix]/[lpCC]+(0.5-[apix]/[lpCC])/[lptimes]*[Ilb12c]
      [lp2fre]=[apix]/[lpCC]+(0.5-[apix]/[lpCC])/[lptimes]*([Ilb12c]+1+0.5)
;
;      [lp1fre]=[apix]/[lpCC]
;      [lp2fre]=[apix]/[lpCC]*2
;
      if([lp1fre].ge.0.5) [lp1fre]=0.5
      if([lp2fre].ge.0.5) [lp2fre]=0.5
;
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;09142009 To test 09142009B Section instead of the following 7 lines.
;      [lp1]=[apix]/[lp1fre]*10
;      [lpmax]=[lp_max]*10
;      if([lp1].ge.[lpmax]) [lp1]=[lpmax]
;      [lp2]=[apix]/[lp2fre]*10
;      if([lp2].ge.[lpmax]) [lp2]=[lpmax]*2/3
;      [lpzp]=[lp1]*2/3
;      if([lp2].lt.[lpzp]) [lp2]=[lpzp]

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;09142009

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;09142009B
;
      [lp1]=[apix]/[lp1fre]*10
      [lpmax]=[lp_max]*10
      if([lp1].ge.[lpmax]) [lp1]=[lpmax]
      [lp2]=[apix]/[lp2fre]*10
      if([lp2].ge.[lpmax]) [lp2]=[lpmax]*2/3
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;09142009B

      sd [Itotal],[lp1],[lp2],[volume_next],[lp_count],[stable_count]
        zz-record-lp
;
;
;      vm
;SPRET-spistack_add_blank.com ${refifolder}/prjtempIP zzmyunselect blankIP
      vm
SPRET-spistack_add_blank.com ${refifolder}/2DRefi001IP zzmyunselect blankIP
;
      vm
SPRET-spistack_mu.com ${refifolder}/2DRefi001IP ${refifolder}/2Dmask{***[volume_next]}IP ${refifolder}/2DRefi001IPmu
;
      vm
SPRET-spistack_norm.com ${refifolder}/2DRefi001IPmu ${refifolder}/2DRefi001IPmunm
      vm
SPRET-spistack_norm.com ${refifolder}/prjtempIP ${refifolder}/prjtempIPnm
;
vm
SPRET-spistack_lpg_forST3DR.com ${refifolder}/prjtempIPnm ${refifolder}/prjtempIPlp ${apix2D} {******[lp1]} {******[lp2]}
      vm
SPRET-spistack_lpg_forST3DR.com ${refifolder}/2DRefi001IPmunm ${refifolder}/2DRefi001IPmulp ${apix2D} {*******[lp1]} {*******[lp2]}
;
; apply mask
;
      do lb77 [xlb12]=1,[nums]
        ud ic,[xlb12],[x20]
          ${selectfile}
; 
        mu
          ${refifolder}/prjtempIPlp@{*******[x20]}
          ${maskfile}IP
          ${refifolder}/prjIPlpmsk@{*******[x20]}
          *
        mu
          ${refifolder}/2DRefi001IPmulp@{*******[x20]}
          ${maskfile}IP
          ${refifolder}/2DRefi001IPmulpmsk@{*******[x20]}
          *
;
      lb77
;
;
      ud ice
        ${selectfile}
;
; calculate shifts
;
      do lb78 [x21]=1,[nums]
        ud ic,[x21],[x20]
          ${selectfile}
;
        CC N
          ${refifolder}/2DRefi001IPmulpmsk@{*******[x20]}
          ${refifolder}/prjIPlpmsk@{*******[x20]}
          _5
;
        pk [x11],[x12]
          _5
          (1)
;
[x13]=[x11]/[IP2D]
[x14]=[x12]/[IP2D]
;
[ax13]=abs([x13])
[ax14]=abs([x14])
;
if([ax13].gt.2) [x13]=0
if([ax14].gt.2) [x14]=0
;
        ud s,[x20],[x111],[x121]
          ${refifolder}/shifts-total{***[Itotal_last]}
;
        sd [x20],[x13],[x14]
          ${refifolder}/shifts{***[Itotal]}
;
        [x112]=[x111]+[x13]
        [x122]=[x121]+[x14]
;
; To check the shifts to make sure them less than [size]/3
;
        [absx112]=abs([x112])
        [absx122]=abs([x122])
;
        [tta]=[size]/3
        if([absx112].gt.[tta]) [x112]=0
        if([absx122].gt.[tta]) [x122]=0
;
        sd [x20],[x112],[x122]
          ${refifolder}/shifts-total{***[Itotal]}
;
[x31]=[x112]*[IP2D]
if([x31].ge.0) [signx31]=1
if([x31].lt.0) [signx31]=-1
;
[x32]=[x122]*[IP2D]
if([x32].ge.0) [signx32]=1
if([x32].lt.0) [signx32]=-1
;
;
[x31]=int([x31]+0.5*[signx31])
[x32]=int([x32]+0.5*[signx32])
;
        SH F
          ${refifolder}/2DRefi000IP@{*******[x20]}
          ${refifolder}/2DRefi001IP@{*******[x20]}
          -[x31],-[x32]
;
;
      lb78
;
      vm
SPRET-spistack_add_blank.com ${refifolder}/2DRefi001IP zzmyunselect blankIP
vm
SPRET-spistack_ip.com ${refifolder}/2DRefi001IP ${refifolder}/2DRefi001 {*****[size]} {*****[size]}
vm
SPRET-spistack_ip.com ${refifolder}/2DRefi001IP ${refifolder}/2DRefi001BP {*****[IP3Dsize]} {*****[IP3Dsize]}
cp
${refifolder}/2DRefi001@
${refifolder}/2Ds{***[Itotal]}@
;
      ud ice
        ${selectfile}
    
;
      sd e
${refifolder}/shifts-total{***[Itotal]}
;
      sd e
${refifolder}/shifts{***[Itotal]}
;
;
vm
SPRET-spistack_add_blank.com ${refifolder}/2DRefi001BP zzmyunselect blankBP
;
vm
SPRET-spistack_mask.com ${refifolder}/2DRefi001BP ${refifolder}/2DRefi001BPmsk ${maskfile}BP
;
vm
SPRET-spistack_ip.com ${refifolder}/2DRefi001BPmsk ${refifolder}/2DRefi001BPmskNormal {*****[size]} {*****[size]}

;
      BP 3f
        ${refifolder}/2DRefi001BPmsk@*******
        ${selectfile}
        ${angfile}
        *
        ${refifolder}/3DRefiBPMask{***[volume_next]}
;
      BP 3f
        ${refifolder}/2DRefi001BPmsk@*******
        ${selectfile}o
        ${angfile}
        *
        ${refifolder}/tmpvol1
;
      BP 3f
        ${refifolder}/2DRefi001BPmsk@*******
        ${selectfile}e
        ${angfile}
        *
        ${refifolder}/tmpvol2

ip
${refifolder}/3DRefiBPMask{***[volume_next]}
${refifolder}/3Ds{***[Itotal]}
[size],[size],[size]
;
mu
${refifolder}/3DRefiBPMask{***[volume_next]}
${refifolder}/3DMask{***[volume_next]}BP
${refifolder}/3DRefiBPMasked{***[volume_next]}
*
vm
proc3d ${refifolder}/3DRefiBPMasked{***[volume_next]}.dat ${refifolder}/3DRefiBPMasked{***[volume_next]}lp.mrc apix=${apix3D} lp={*****[lp_thr]}

cp from ccp4
${refifolder}/3DRefiBPMasked{***[volume_next]}lp.mrc
${refifolder}/3DRefiBPMasked{***[volume_next]}lp
n
;
[volume_end_voxels3BP]=[volume_end_voxels3]*[IP3D]*[IP3D]*[IP3D]
;
fv [thr_cen]
${refifolder}/3DRefiBPMasked{***[volume_next]}lp
[volume_end_voxels3BP]

cg 3 [cx],[cy],[cz]
${refifolder}/3DRefiBPMasked{***[volume_next]}lp
[thr_cen]
;
fs [thrmax]
${refifolder}/3DRefiBPMasked{***[volume_next]}lp

if([thrmax].lt.[thr_cen]) then
[thrzz]=[thrmax]/2
cg 3 [cx],[cy],[cz]
${refifolder}/3DRefiBPMasked{***[volume_next]}lp
[thrzz]
endif

sh f
${refifolder}/3DRefiBPMasked{***[volume_next]}
${refifolder}/3DRefiBPMasked{***[volume_next]}sh
-[cx],-[cy],-[cz]


      pj 3q
        ${refifolder}/3DRefiBPMasked{***[volume_next]}sh
        [hsizeBP]
        [first]-[last]
        ${angfile}
        ${refifolder}/prjtempBP@*******
;

ip
${refifolder}/tmpvol1
${refifolder}/tmpvol1sk
[size],[size],[size]
;
ip
${refifolder}/tmpvol2
${refifolder}/tmpvol2sk
[size],[size],[size]


      rf 3 [fsc_pix] [fsc_frq]
        ${refifolder}/tmpvol1sk
        ${refifolder}/tmpvol2sk
        (1)
        (0.6,1.2)
        W
        [maxang]
        (3.0)
        ${output}-rf3doc
;
      rf 3 [fsc_pix] [fsc_frq]
        ${refifolder}/tmpvol1sk
        ${refifolder}/tmpvol2sk
        (1)
        (0.6,1.2)
        W
        [maxang]
        (3.0)
        ${refifolder}/rf3doc{***[Itotal]}
;
;      vm
;SPRET-spistack_add_blank.com ${refifolder}/prjtempBP zzmyunselect blankBP
vm
SPRET-spistack_ip.com ${refifolder}/prjtempBP ${refifolder}/prjtempIP {*****[IP2Dsize]} {*****[IP2Dsize]}
vm
SPRET-spistack_ip.com ${refifolder}/prjtempBP ${refifolder}/prjtempNoIP {*****[size]} {*****[size]}
;
;
      [lpCC]=1/[fsc_frq]*[apix]*1.5
      [lpCCcut]=[lpCC]*2/3
[real_resolution]=1/[fsc_frq]*[apix]
;
[lp_count]=0
      sd [Itotal],[fsc_pix],[fsc_frq],[real_resolution],[volume_next],[lp_count],[stable_count],[lp1],[lp2]
        ${output}-record-fsc
;
      vm
SPRET-doc_shift_statistics_refi.com 1 zz-templast-2 ${refifolder}/shifts-total{***[Itotal]}
;
      ud s,[one],[absPerSet],[absPerImg]
        zz-templast-2
;
      sd [Itotal],[absPerSet],[absPerImg],[volume_next],[lp_count],[stable_count]
        ${output}-record-shift-statistics
;
; To check the creteria: shift changes
;
        [last_shift4]=[Itotal]-0
        [last_shift3]=[Itotal]-1
        [last_shift2]=[Itotal]-2
        [last_shift1]=[Itotal]-3
;
        if([last_shift4].lt.0) [last_shift4]=0
        if([last_shift3].lt.0) [last_shift3]=0
        if([last_shift2].lt.0) [last_shift2]=0
        if([last_shift1].lt.0) [last_shift1]=0
vm
SPRET-doc_shift_statistics_refi.com 4 zz-temp-shift_sta1 ${refifolder}/shifts-total{***[last_shift1]} ${refifolder}/shifts-total{***[last_shift2]} ${refifolder}/shifts-total{***[last_shift3]} ${refifolder}/shifts-total{***[last_shift4]}
;
        vm
SPRET-doc_shift_statistics_refi.com 1 zz-temp-shift_sta2 ${refifolder}/shifts-total{***[last_shift4]}
        vm
SPRET-doc_shift_statistics_refi.com 1 zz-temp-shift_sta3 ${refifolder}/shifts{***[last_shift4]}
;
vm
cp zz-temp-shift_sta1.dat zz-temp-shift_sta11.dat
vm
cp zz-temp-shift_sta2.dat zz-temp-shift_sta21.dat
vm
cp zz-temp-shift_sta3.dat zz-temp-shift_sta31.dat
;
if([stable_count].ge.4) then
        ud s,[one],[absPerSet1],[absPerImg1]
          zz-temp-shift_sta11
        ud s,[one],[absPerSet2],[absPerImg2]
          zz-temp-shift_sta21
        ud s,[one],[absPerSet3],[absPerImg3]
          zz-temp-shift_sta31
;
if([absPerImg2].gt.0) [absChange]=abs(([absPerImg1]-[absPerImg2])/[absPerImg2])
if([absPerImg2].eq.0) [absChange]=0
;
sd [Itotal],[absPerSet1],[absPerImg1],[absPerSet2],[absPerImg2],[absChange]
${output}-record-shift-change1
sd [Itotal],[absPerImg3],[volume_next],[lp_count],[stable_count]
${output}-record-shift-change2
; 
        [judge1]=0
        if([absPerImg3].lt.0.001) [judge1]=1
        [judge2]=0
        if([absChange].le.0.01) [judge2]=1
        [judge3]=[judge1]*[judge2]
        if([judge3].eq.1) [ifstable]=1
;
;        if([absPerImg3].lt.0.001) then
;           if([absChange].le.0.01) then
;              [ifstable]=1
;           endif
;        endif
endif
;
LB39
lb44



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; IP and find more accurate shifts End
;
sd e
${output}-record-shift-change1
sd e
${output}-record-shift-change2
sd e
${output}-record-fsc
sd e
${output}-record-shift-statistics
;
IP
${refifolder}/3DRefiBPMask{***[volume_next]}
${output}-loop3
[size],[size],[size]
;
cp
${refifolder}/prjtempNoIP@
${output}-loop3-prj@
;
cp
${refifolder}/2DRefi001BPmskNormal@
${output}-loop3-2Ds@
;
vm
proc3d ${output}-loop3.dat ${output}-loop3-lp8.mrc apix=${apix} lp=8
;
;
vm
SPRET-spistack_mixture.com 8 ${totalNum} ${output}-mixture ${output}-loop0-2Ds ${output}-loop0-prj ${output}-loop1-2Ds ${output}-loop1-prj ${output}-loop2-2Ds ${output}-loop2-prj ${output}-loop3-2Ds ${output}-loop3-prj
en d
;
eot
# rm -r ${refifolder}
rm LOG.dat
rm ${output}tempo.dat
rm ${output}tempe.dat
rm blank.dat
rm zztemplp.dat zztemplp.mrc
