#!/bin/csh
#
#
#   To auto generate box files based on the given box step by step. Each step's box is calculated based on last step's box.
#
#   usage: SPRET-autotracebox-step-adv.com [mrc_file_prefix] [total_image] [angle_step] [img_size] [ID(ref_box)] [ID(untilt)] [first] [last] [bright] [contrast] [scale]
#
#      where:
#
#          [mrc_file_prefix]: the prefix of the input mrc files.
#
#          [total_image]:     numbers of the images to be processed.
#
#          [angle_step]:      tilt angle difference between two neighbouring images (positive or negative).
#
#          [img_size]:        the size of each image.
#
#          [ID(ref_box)]:     the index of the reference box.
#
#          [ID(untilt):       the index of the untilted box (which have zero degree tilt angle, in most cases, it is the middle one of the whole stack).
#
#          [first]:           which one is the beginning of the images to be processed.
#
#          [last]:            the last one to be processed of the images.
#
#          [bright]:          bright value in EMAN boxer.
#
#          [contrast]:        contrast value in EMAN boxer.
#
#          [scale]:           scale value in EMAN boxer.
#
#
#
#   Lei Zhang @ REN LAB, UCSF, Mon Feb  2 12:36:21 PST 2009
#
#   SPRET-autotrace-tiltbox-call.com, SPRET-autotrace-tiltbox are needed.
#
#   --------- ver 02, update to track and calculate box step by step
#            Gang Ren, Wed Feb  4 15:42:40 PST 2009
#
#
#    alias MATH 'set \!:1 = `echo "\!:3-$" | bc -l`'
#
#    Simply trade the [step] as a abs([step]) value for the symmetric of Cos() function. Lei Zhang @ REN Lab, UCSF, Wed May 13 2009.
#
#
#
#---------------------------------------------------------------------------------------------------------
if ($#argv != 11) then
  echo "To auto generate box files based on the given box"
  echo "usage: SPRET-autotracebox-step-adv.com [mrc_file_prefix] [total_image] [angle_step] [img_size] [ID(ref_box)] [ID(untilt)] [first] [last] [bright] [contrast] [scale]"
  echo "such as: SPRET-autotracebox-step-adv.com tomoCTF-00_ 95 1.5 2048 48 48 1 95 -0.1 0.5 3"
  exit
endif
#
set mrcpre=$1
set totaln=$2
set angstep=$3
set imgsize=$4
set IDref=$5
set IDuntilt=$6
set first=$7
set last=$8
set bright=$9
set contrast=$10
set scale=$11
#
# MATH halfsize = ${IDuntilt} / 2
spider dat << zl
[Nums]=${totaln}
[step]=${angstep}
;Simply trade the [step] as a abs([step]) value for the symmetric of Cos() function.
[step]=abs([step])
[id_refbox]=${IDref}
[untilt]=${IDuntilt}
[first]=${first}
[last]=${last}
;
if([id_refbox].eq.[first]) en
if([id_refbox].eq.[last]) en
;
[prevbox]=[id_refbox]
;
[ref_tilt]=abs(-1*([id_refbox]-[untilt])*[step]*10)
;
if([first].le.[last]) then
  ;
  do lb1 [xlb1]=[first],[last]
    [tilt]=abs(([xlb1]-[untilt])*[step]*10)
    vm
    SPRET-autotrace-tiltbox-call.com ${mrcpre}{****[prevbox]}.box ${mrcpre}{****[xlb1]}.box ${imgsize} {****[tilt]} 0 0 {****[ref_tilt]}
    vm
    boxer ${mrcpre}{****[xlb1]}.mrc box=${mrcpre}{****[xlb1]}.box scale=${scale} bright=${bright} contrast=${contrast}
    [prevbox]=[xlb1]
    [ref_tilt]=abs(([xlb1]-[untilt])*[step]*10)
  lb1
  ;
endif
if([first].gt.[last]) then
  ;
  do lb2 [xlb1]=[first],[last],-1
    [tilt]=abs(([xlb1]-[untilt])*[step]*10)
;    [ref_tilt]=abs(-1*([id_refbox]-[untilt])*[step]*10)
    vm
    SPRET-autotrace-tiltbox-call.com ${mrcpre}{****[prevbox]}.box ${mrcpre}{****[xlb1]}.box ${imgsize} {****[tilt]} 0 0 {****[ref_tilt]}
    vm
    boxer ${mrcpre}{****[xlb1]}.mrc box=${mrcpre}{****[xlb1]}.box scale=${scale} bright=${bright} contrast=${contrast}
    [prevbox]=[xlb1]
    [ref_tilt]=abs(([xlb1]-[untilt])*[step]*10)

  lb2
  ;
endif
;
en d
zl

rm LOG.dat
