#!/bin/csh
#
#
#   Pick up the particles from box files.
#
#   usage: SPRET-autotrace-parselect.com [par_series_nums] [img_prefix] [box_prefix] [first] [last] [mask_file_no_ext] [output_file_prefix] [Apix] [hp_cut(A)]
#
#      where:
#
#          [par_series_nums]:    number of particles in each box.
#     
#          [img_prefix]:         mrc images' prefix.
#
#          [box_prefix]:         box files' prefix.
#
#          [first]:              starting index of the mrc files to be boxed.
#
#          [last]:               end index of the mrc files to be boxed.
#
#          [mask_file_no_ext]:   mask file without the extension.
#
#          [output_file_prefix]: output files' prefix.
#
#          [Apix]:               A/pix.
#
#          [hp_cut(A)]:          High-pass filter's radius in the unit of A.
#
#
#
#   Three output files are generated:
#      output_par*.dat --> raw data;
#      output_par*nm.dat --> normalization of raw data, high-pass filter is applied.
#      output_par*masknm.dat --> masked and normlized raw data, high-pass filter is applied.
#
#   Mon Feb  2 14:28:41 PST 2009, Zhang Lei @ REN Lab, UCSF
#
#-----------------------------------------------------------------------------------------------
if ($#argv != 9) then
  echo "select the particles from boxered files"
  echo "usage: SPRET-autotrace-parselect.com [par_series_nums] [img_prefix] [box_prefix] [first] [last] [mask_file_no_ext] [output_file_prefix] [Apix] [hp_cut(A)]"
  echo "such as: SPRET-autotrace-parselect.com 13 tomoCTF-00_ tomoCTF-00_ 1 95 mask_fq tomoCTF00- 2.812 500"
  exit
endif
#
set parnums=$1
set imgpre=$2
set boxpre=$3
set first=$4
set last=$5
set mask=$6
set output=$7
set Apix=$8
set hp=$9
#
rm par_all.dat
spider dat << zl
[all_idx]=0
do lb1 [x0]=1,${parnums}
vm
rm -r par{****[x0]}
vm
mkdir par{****[x0]}
[x00]=[x0]-1
vm
rm par{****[x0]}*
;
;To pick up the particles, each particle is stored in par****/ directory.
;
do lb2 [x1]=${first},${last}
[all_idx]=[all_idx]+1
vm
rm zzjjj* zzs*
vm
batchboxer input=${imgpre}{****[x1]}.mrc dbbox=${boxpre}{****[x1]}.box output=zzjjj.hed 
#newsize=384
;
;To do the high-pass filter.
;
vm
proc2d zzjjj.hed zzjjj.mrc first={****[x00]} last={****[x00]} apix=${Apix} hp=${hp}
;
;To do the normalization.
;
vm
proc2d zzjjj.mrc zzs.mrc norm=0,10
cp from ccp4
zzjjj.mrc
_1
N
cp
_1
par{****[x0]}/${output}par{****[x0]}@{*******[x1]}
cp
_1
par_all@{*******[all_idx]}
cp from ccp4
zzs.mrc
_2
N
cp
_2
par{****[x0]}/${output}par{****[x0]}nm@{*******[x1]}
;
;To apply the mask, and normalization again.
;
mu
_2
${mask}
zzjjj2
*
;
vm
proc2d zzjjj2.dat zzjjj2.mrc norm=0,10
cp from ccp4
zzjjj2.mrc
zzjjj3
N
cp
zzjjj3
par{****[x0]}/${output}par{****[x0]}msknm@{*******[x1]}
vm
rm zzjjj* zzs*
lb2
lb1
en
zl
rm results.dat*
