#!/bin/csh
#
#
#   Do the backprojection based on the input Spider stack file, angle file, and select file.
#
#   Users have the choice to do a additional refinement.
#
#   And in the refinement process, crosscorrelation is used to center 2D images. 
#
#   usage: SPRET-backprojection_forST3DR.com [input_stack_no_ext] [size(pix)] [angle_file_noext] [select_file_noext] [output_prefix] [if_refine: 0: norefi, >0 refine times] [apix] [mask_no_ext]
#
#      where:
#
#          [input_stack_no_ext]: the input Spider stack file to do backprojection.
#     
#          [size(pix)]:          each image's size, in the unit of pixel.
#
#          [angle_file_noext]:   the file contains Euler Angles for the backprojection, could be created by SPRET-create_angle.com, and without extension .dat.
#
#          [select_file_noext]:  the file contains particle selection for the backprojection, could be created by SPRET-create_select.com, and without extension .dat.
#
#          [output-prefix]:      the prefix of the output boxes.
#
#          [if_refine]:          0: no refinement, >0 refinement times.
#
#          [Apix]:               A/pix.
#
#          [mask_no_ext]:        the mask file same as the one which is applied to input stack, for the purpose of making same mask as the input ones in the *mixture.dat.
#
#	Lei Zhang @ REN Lab, UCSF, Thu Feb  13 17:19:34 PST 2009
#
#   Specialized for the request of SPRET-ST3DR.com
#
#   Revise [hsize] from int([size]/2)-2 to int([size]/2), Feb 20 2009 Lei
#
#   Add mask input file for prj, Feb 20 2009 Lei
#
#-------------------------------------------------------------------------------------------------------
if ($#argv != 8) then
  echo "Do the backprojection based on the input Spider stack file, angle file, and select file."
  echo "usage: SPRET-backprojection_forST3DR.com [input_stack_no_ext] [size(pix)] [angle_file_noext] [select_file_noext] [output_prefix] [if_refine: 0: norefi, >0 refine times] [apix] [mask_no_ext]"
  echo "such as: SPRET-backprojection_forST3DR.com par0001msknm 64 angcal myselect Tomo01-par00013D_ 1 2.812 ../mask_fq"
  exit
endif
#
#
set input=$1
set size=$2
set angfile=$3
set selectfile=$4
set output=$5
set ifrefi=$6
set Apix=$7
set mask=$8
#
#
#
SPRET-create_select_odd.com ${selectfile} ${selectfile}o
SPRET-create_select_even.com ${selectfile} ${selectfile}e
#
spider dat << eot
;
[ifrefi]=${ifrefi}
[size]=${size}
;[hsize]=int([size]/2)-2
[hsize]=int([size]/2)
[first]=1
[one]=1
ud n,[last]
${angfile}
;
ud s,[one],[ztemp],[maxang]
${angfile}
;
[maxang]=abs([maxang])
;
vm
SPRET-spistack_mask.com ${input} ${input}msk ${mask}
;
bp 3f
${input}msk@*******
${selectfile}
${angfile}
*
${output}
;${output}tempo
;${output}tempe
bp 3f
${input}msk@*******
${selectfile}o
${angfile}
*
${output}tempo
;
;
bp 3f
${input}msk@*******
${selectfile}e
${angfile}
*
${output}tempe
;
rf 3
${output}tempo
${output}tempe
(1)
(0.6,1.2)
W
[maxang]
(3.0)
${output}-rf3doc
;
vm
proc3d ${output}.dat ${output}.mrc apix=${Apix} mrc
;
vm
proc3d ${output}.dat ${output}-lp8.mrc apix=${Apix} lp=8
;
vm
proc3d ${output}.dat ${output}-lp16.mrc apix=${Apix} lp=16
;
vm
proc3d ${output}.dat ${output}-lp26.mrc apix=${Apix} lp=26
;
vm
proc3d ${output}.dat ${output}-lp36.mrc apix=${Apix} lp=36
;
vm
proc3d ${output}.dat ${output}-lp46.mrc apix=${Apix} lp=46
;
if([ifrefi].gt.0) then
; REFINEMENT
vm
rm -r refivolzz
vm
mkdir refivolzz
; SPRET-3D_lp.com [input_3D_no_ext] [output_3D_no_ext] [a/pix] [lp(A)]"
vm
SPRET-3D_lp.com ${output} refivolzz/opttemp ${Apix} 30
vm
SPRET-3D_norm.com refivolzz/opttemp refivolzz/opttempnm ${Apix}
;cg 3 [xx21],[xx22],[xx23],[xxr]
;refivolzz/opttempnm
;3
;[xx21]=0
;[xx22]=0
;sh f
;${output}
;refivolzz/voltemp1
;-[xx21],-[xx22],-[xx23]
;cp
;refivolzz/voltemp1
;refivolzz/3DRefi001
;
cp
${output}
refivolzz/3DRefi001
ud n,[nums]
${selectfile}
do lb23 [x76]=1,${ifrefi}
[x86]=[x76]+1
; use crosscorrelation to center 2D images
; calculate projections of 3D in the directions of original projections
;center 3D
;cg 3 [xx21],[xx22],[xx23],[xxr]
;refivolzz/3DRefi{***[x76]}
;0.5
;[xx21]=0
;[xx22]=0
;sh f
;refivolzz/3DRefi{***[x76]}
;refivolzz/voltemp1
;-[xx21],-[xx22],-[xx23]
;cp
;refivolzz/voltemp1
;refivolzz/3DRefi{***[x76]}
;
;
pj 3q
refivolzz/3DRefi{***[x76]}
[hsize]
${selectfile}
${angfile}
refivolzz/prjtempNnm@*******
vm
SPRET-spistack_norm.com refivolzz/prjtempNnm refivolzz/prjtemp
;
;
; apply mask
do lb24 [xlb24]=1,[nums]
ud s,[xlb24],[x20]
${selectfile}
mu
refivolzz/prjtemp@{*******[x20]}
${mask}
refivolzz/prj@{*******[x20]}
*
lb24
;
;
; calculate shifts
do lb13 [x21]=1,[nums]
ud ic,[x21],[x20]
${selectfile}
;
CC N
${input}@{*******[x20]}
refivolzz/prj@{*******[x20]}
_5
pk [x11],[x12]
_5
(1)
;or sh [xrot],[xs],[ys],[c3],[c4]


;
;
SH F
${input}@{*******[x20]}
refivolzz/tmp@{*******[x20]}
-[x11],-[x12]
;
;
mu
refivolzz/tmp@{*******[x20]}
${mask}
refivolzz/tmpmsk@{*******[x20]}
*
;
; store the shift parameters
sd [x20],[x11],[x12]
refivolzz/shifts{***[x86]}
lb13
ud ice
${selectfile}
sd e
refivolzz/shifts{***[x86]}
BP 3f
refivolzz/tmpmsk@*******
${selectfile}
${angfile}
*
refivolzz/3DRefi{***[x86]}
;refivolzz/tmpvol1
;refivolzz/tmpvol2
;
;
BP 3f
refivolzz/tmp@*******
${selectfile}o
${angfile}
*
refivolzz/tmpvol1
;
BP 3f
refivolzz/tmp@*******
${selectfile}e
${angfile}
*
refivolzz/tmpvol2
;
;
; center 3D
;cg ph,x10,x11,x12,x21,x22,x23
;refi{**X78}/tvol{***x86}
;sh 3
;refi{**X78}/tvol{***x86}
;refi{**X78}/vol{***x86}
;-x21,-x22
;-x23
[one]=1
ud s,[one],[ztemp],[maxang]
${angfile}
;
[maxang]=abs([maxang])
;
vm
rm zzrf3doc.dat
rf 3
refivolzz/tmpvol1
refivolzz/tmpvol2
(1)
(0.6,1.2)
W
[maxang]
(3.0)
${output}-rf3doc
lb23
;
vm
cp refivolzz/shifts{***[x86]}.dat ${output}-shiftsdata.dat
vm
proc3d refivolzz/3DRefi{***[x86]}.dat refivolzz/3DRefi{***[x86]}.mrc apix=${Apix} mrc
;
vm
proc3d refivolzz/3DRefi{***[x86]}.dat refivolzz/3DRefilp8{***[x86]}.mrc apix=${Apix} lp=8
vm
proc3d refivolzz/3DRefi{***[x86]}.dat refivolzz/3DRefilp16{***[x86]}.mrc apix=${Apix} lp=16
vm
proc3d refivolzz/3DRefi{***[x86]}.dat refivolzz/3DRefilp26{***[x86]}.mrc apix=${Apix} lp=26
vm
proc3d refivolzz/3DRefi{***[x86]}.dat refivolzz/3DRefilp36{***[x86]}.mrc apix=${Apix} lp=36
vm
proc3d refivolzz/3DRefi{***[x86]}.dat refivolzz/3DRefilp46{***[x86]}.mrc apix=${Apix} lp=46
vm
cp refivolzz/3DRefi{***[x86]}.mrc ${output}-Refi.mrc
vm
cp refivolzz/3DRefi{***[x86]}.dat ${output}-Refi.dat
vm
cp refivolzz/tmpvol1.dat ${output}-Refi-o.dat
vm
cp refivolzz/tmpvol2.dat ${output}-Refi-e.dat
vm
cp refivolzz/3DRefilp8{***[x86]}.mrc ${output}-Refi-lp8.mrc
;
vm
cp refivolzz/3DRefilp16{***[x86]}.mrc ${output}-Refi-lp16.mrc
;
vm
cp refivolzz/3DRefilp26{***[x86]}.mrc ${output}-Refi-lp26.mrc
vm
cp refivolzz/3DRefilp36{***[x86]}.mrc ${output}-Refi-lp36.mrc
vm
cp refivolzz/3DRefilp46{***[x86]}.mrc ${output}-Refi-lp46.mrc
;
vm
cp refivolzz/prj.dat ${output}-Refi-prj.dat
;
vm
cp refivolzz/tmp.dat ${output}-Refi-shift.dat
;
vm
cp zztemprf3doc.dat ${output}-Refi-rf3doc.dat
;
;
bl
blank
[size],[size]
n
0
;
;
do lb93 [xlb33]=[first],[last]
[judge]=0
do lb94 [xlb34]=1,[nums]
ud ic,[xlb34],[xlb35]
${selectfile}
if([xlb35].eq.[xlb33]) [judge]=1
lb94
if([judge].eq.0) then
cp
blank
${output}-Refi-prj@{*******[xlb33]}
cp
blank
${output}-Refi-shift@{*******[xlb33]}
endif
lb93
;
;
;
;de
;${output}-mixture
;
;SPRET-spistack_mixture.com [total_nums_of_input_stacks] [number_of_images_in_each_stack] [output_no_ext] [input1_no_ext] [input2_no_ext] [input3_no_ext] [input4_no_ext] ... ...
;vm
;SPRET-spistack_mixture.com 3 {*****[last]} ${output}-mixture ${input} ${output}-Refi-prj ${output}-Refi-shift
;
;
ud ice
${selectfile}
endif
en
eot
# rm -r refivolzz
rm results.dat* 
rm ${output}tempo.dat
rm ${output}tempe.dat
rm blank.dat
