#!/bin/csh
#
#
#   Do the backprojection based on the input Spider stack file, angle file, and select file. Several iterations incluing normal circular 2D mask, and low-pass filter are applied.
#
#
#   "usage: SPRET-backprojection_forST3DR_onlyRoundOne.com [input_stack_no_ext] [output_prefix] [angle_file_noext] [select_file_noext] [apix] [mask_no_ext] [protein_molecular_weight(KD)]
#
#      where:
#
#          [input_stack_no_ext]: the input Spider stack file to do backprojection.
#     
#          [output-prefix]:      the prefix of the output 3D volumes.
#
#          [angle_file_noext]:   the file contains Euler Angles for the backprojection, could be created by SPRET-create_angle.com, and without extension .dat.
#
#          [select_file_noext]:  the file contains particle selection for the backprojection, could be created by SPRET-create_select.com, and without extension .dat.
#
#          [apix]:               a/pix.
#
#          [mask_no_ext]:        the mask file same as the one which is applied to input stack, for the purpose of making same mask as the input ones in the *mixture.dat and in the refinement processes.
#
#
#	Lei Zhang @ REN Lab, UCSF, Jul. 15 PST 2009
#
#   Specialized for the request of SPRET-ST3DR.com
#
#---------------------------------------------------------------------------------------------------------------------------------------
#
if ($#argv != 7) then
  echo "Do the backprojection based on the input Spider stack file, angle file, and select file."
  echo "usage: SPRET-backprojection_forST3DR_onlyRoundOne-fixCenter.com [input_stack_no_ext] [output_prefix] [angle_file_noext] [select_file_noext] [apix] [mask_no_ext] [protein_molecular_weight(KD)]"
  echo "such as: SPRET-backprojection_forST3DR_onlyRoundOne-fixCenter.com par0001msknm Tomo01-par0001 myang mysel 2.812 ../mask_fq 208"
  exit
endif
#
#
rm -r /tmp/eman2*
rm -r ~/.eman2
#
alias MATH 'set \!:1 = `echo "\!:3-$" | bc -l`'
#
set input=$1
set output=$2
#set size=$2
set angfile=$3
set selectfile=$4
set apix=$5
set maskfile=$6
set KD=$7
set folder_exist = `ls -dir refivolzz* | wc -l`
@ folderNum = ${folder_exist} + 1
set i3=`printf "%03d" ${folderNum}`
set totNum1 = `more ${angfile}.dat | wc -l`
@ totalNum = ${totNum1} - 1
set refifolder = "refivolzzz"${i3}
set inputlp=${input}lp
#
SPRET-create_select_odd.com ${selectfile} ${selectfile}o
SPRET-create_select_even.com ${selectfile} ${selectfile}e
SPRET-create_select.com zz-sel-all 1-${totalNum}
#
#
spider dat << eot
;
;Read parameters from input strings.
;
[apix]=${apix}
;
[KD]=${KD}
;

;
[one]=1
[zero]=0
;
fi [size]
  ${input}@0000001
  2
;
;
[first]=1
;
ud n,[nums]
  ${selectfile}
;
ud n,[last]
  ${angfile}
;
ud s,[last],[ztemp],[maxang]
  ${angfile}
;
[maxang]=abs([maxang])
;
[hsize]=int([size]/2)
;
[lp_max]=[size]/6*[apix]
;
[ffqv10]=[apix]*[size]/10
[lpdivd4]=[apix]*[size]/4
;
; Create blank file.
;
mo
  blank
  [size],[size]
  b
  0.0
;
vm
SPRET-create_unselect.com ${selectfile} zzmyunselect {****[last]}
;
; At first, completely back projection of the raw data without any tricks.
;
vm
SPRET-spistack_mask.com ${input} ${input}msk ${maskfile}
bp 3f
  ${input}msk@*******
  ${selectfile}
  ${angfile}
  *
  ${output}
;
bp 3f
  ${input}msk@*******
  ${selectfile}o
  ${angfile}
  *
  ${output}tempo
;
bp 3f
  ${input}msk@*******
  ${selectfile}e
  ${angfile}
  *
  ${output}tempe
;
[fsc_count]=0
;
rf 3 [fsc_pix] [fsc_frq]
  ${output}tempo
  ${output}tempe
  (1)
  (0.6,1.2)
  W
  [maxang]
  (3.0)
  ${output}-rf3doc
;
;
[resolution]=int(1/[fsc_frq]*[apix])
[resolution2]=[resolution]*2
;
vm
proc3d ${output}.dat ${output}.mrc apix=${apix} mrc
;
vm
proc3d ${output}.dat ${output}-lp008.mrc apix=${apix} lp=8
;
vm
proc3d ${output}.dat ${output}-lp{***[resolution]}.mrc apix=${apix} lp={****[resolution]}
;
vm
proc3d ${output}.dat ${output}-lp{***[resolution2]}.mrc apix=${apix} lp={****[resolution2]}
;
;
de 
  ${output}-record-fsc
de
  ${output}-record-shift-statistics
de
  ${output}-record-shift-change1
de
  ${output}-record-shift-change2
;
;
[real_resolution]=1/[fsc_frq]*[apix]
;
sd [one],[fsc_pix],[fsc_frq],[real_resolution]
  ${output}-record-fsc
;

[lpCC]=1/[fsc_frq]*[apix]*1.5
[lpCCcut]=[lpCC]*2/3
[lpnormal]=1/[fsc_frq]*[apix]*1.5
;
;
vm
rm -r ${refifolder}
vm
mkdir ${refifolder}
;
vm
cp ${output}-rf3doc.dat ${refifolder}/rf3doc000.dat
vm
cp ${input}.dat ${refifolder}/2DRefi000.dat
vm
cp ${input}.dat ${refifolder}/2Ds000.dat
;
cp
${output}
${refifolder}/3DRefi000
;
cp
${output}
${refifolder}/3Ds000
;
do lb51 [x1]=1,[nums]
  ud s,[x1],[xxx]
    ${selectfile}
  sd [xxx],[zero],[zero]
    ${refifolder}/shifts-total000
lb51
  sd e
${refifolder}/shifts-total000
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;First round, no 3D mask, no 2D mask. Begin
;
;
[Itotal]=0
[volume_last]=-1
[volume_next]=0
;
do lb01 [Ilb01]=1,1
;
  [volume_last]=[volume_last]+1
  [volume_next]=[volume_next]+1
;
  pj 3q
    ${refifolder}/3DRefi{***[volume_last]}
    [hsize]
    [first]-[last]
    ${angfile}
    ${refifolder}/prjtemp@*******
;
;    vm
;SPRET-spistack_add_blank.com ${refifolder}/prjtemp zzmyunselect blank
    vm
SPRET-spistack_add_blank.com ${refifolder}/2DRefi000 zzmyunselect blank
;
;  [I_lp_num]=int([lpCC]/[apix])
;
  [factor]=0.5
;
  vm
cp ${refifolder}/2DRefi000.dat ${refifolder}/2DRefi001.dat
;
;  [lptimes]=10  To test the lptimes from 10 to 20 09142009
  [lptimes]=20
;
  [lp_count]=0
;
  [if_lpstop]=0
;
  do lb02 [Ilb02]=0,[lptimes]
;  do lb02 [Ilb02]=0,1
;
    [lp_count]=[lp_count]+1
    if([if_lpstop].eq.1) goto LB92
;
    if([Ilb02].ge.2) then
vm
cp ${output}-record-fsc.dat ${output}-record-fsctemp.dat
    ud s,[Itotal],[fsc_pix_0],[fsc_frq_0],[real_resolution_0]
      ${output}-record-fsctemp
    endif
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
    [ifstable]=0
;
    [stable_count]=0
;
    do lb03 [Ilb03]=0,60
;
      [stable_count]=[stable_count]+1
;
      if([ifstable].gt.0) goto LB91
;output
      [Itotal_last]=[Itotal]
      [Itotal]=[Itotal]+1
;
      [lp1fre]=[apix]/[lpCC]+(0.5-[apix]/[lpCC])/[lptimes]*[Ilb02]
;      [lp2fre]=[apix]/[lpCC]+(0.5-[apix]/[lpCC])/[lptimes]*([Ilb02]+1)  This line is blocked as to test the next line. 09142009.
      [lp2fre]=[apix]/[lpCC]+(0.5-[apix]/[lpCC])/[lptimes]*([Ilb02]+1+0.5)
;
;      [lp1fre]=[apix]/[lpCC]
;      [lp2fre]=[apix]/[lpCC]*2
;
      if([lp1fre].ge.0.5) [lp1fre]=0.5
      if([lp2fre].ge.0.5) [lp2fre]=0.5
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;09142009 To test 09142009B Section instead of the following 7 lines.
;      [lp1]=[apix]/[lp1fre]*10
;      [lpmax]=[lp_max]*10
;      if([lp1].ge.[lpmax]) [lp1]=[lpmax]
;      [lp2]=[apix]/[lp2fre]*10
;      if([lp2].ge.[lpmax]) [lp2]=[lpmax]*2/3
;      [lpzp]=[lp1]*2/3
;      if([lp2].lt.[lpzp]) [lp2]=[lpzp]

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;09142009

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;09142009B
;
      [lp1]=[apix]/[lp1fre]*10
      [lpmax]=[lp_max]*10
      if([lp1].ge.[lpmax]) [lp1]=[lpmax]
      [lp2]=[apix]/[lp2fre]*10
      if([lp2].ge.[lpmax]) [lp2]=[lpmax]*2/3
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;09142009B


;
      sd [Itotal],[lp1],[lp2],[volume_next],[lp_count],[stable_count]
        zz-record-lp
;
;      vm
;SPRET-spistack_add_blank.com ${refifolder}/prjtemp zzmyunselect blank
      vm
SPRET-spistack_add_blank.com ${refifolder}/2DRefi001 zzmyunselect blank
      vm
SPRET-spistack_norm.com ${refifolder}/prjtemp ${refifolder}/prjtempnm
      vm
SPRET-spistack_lpg_forST3DR.com ${refifolder}/prjtempnm ${refifolder}/prjtemplp ${apix} {******[lp1]} {******[lp2]}
      vm
SPRET-spistack_lpg_forST3DR.com ${refifolder}/2DRefi001 ${refifolder}/2DRefi001lp ${apix} {*******[lp1]} {*******[lp2]}
;
; apply mask
;
      do lb52 [xlb12]=1,[nums]
        ud ic,[xlb12],[x20]
          ${selectfile}
; 
        mu
          ${refifolder}/prjtemplp@{*******[x20]}
          ${maskfile}
          ${refifolder}/prjlpmsk@{*******[x20]}
          *
        mu
          ${refifolder}/2DRefi001lp@{*******[x20]}
          ${maskfile}
          ${refifolder}/2DRefi001lpmsk@{*******[x20]}
          *
;
      lb52
;
;
      ud ice
        ${selectfile}
;
; calculate shifts
;
      do lb53 [x21]=1,[nums]
        ud ic,[x21],[x20]
          ${selectfile}
;
        CC N
          ${refifolder}/2DRefi001lpmsk@{*******[x20]}
          ${refifolder}/prjlpmsk@{*******[x20]}
          _5
;
        pk [x11],[x12]
          _5
          (1)
        sd [x20],[x11],[x12]
          ${refifolder}/shifts{***[Itotal]}
;
        ud ic,[x21],[x20]
          ${selectfile}
;
        ud s,[x20],[x111],[x121]
          ${refifolder}/shifts-total{***[Itotal_last]}
;
        [x112]=[x111]+[x11]
        [x122]=[x121]+[x12]
;
; To check the shifts to make sure them less than [size]/3
;
        [absx112]=abs([x112])
        [absx122]=abs([x122])
;
        [tta]=[size]/3
        if([absx112].gt.[tta]) [x112]=0
        if([absx122].gt.[tta]) [x122]=0
;
        sd [x20],[x112],[x122]
          ${refifolder}/shifts-total{***[Itotal]}
;
;        SH F
;          ${refifolder}/2DRefi000@{*******[x20]}
;          ${refifolder}/2DRefi001@{*******[x20]}
;          -[x112],-[x122]
;       ud s,[x20],[x11],[x12]
;         ${refifolder}/shifts{***[Itotal]}
        SH F
          ${refifolder}/2DRefi000@{*******[x20]}
          ${refifolder}/2DRefi001@{*******[x20]}
          -[x112],-[x122]

;
      lb53
;
cp
${refifolder}/2DRefi001@
${refifolder}/2Ds{***[Itotal]}@
      ud ice
        ${selectfile}
    
;
      sd e
${refifolder}/shifts{***[Itotal]}
      sd e
${refifolder}/shifts-total{***[Itotal]}
;
; Apply mask to data before them backprojection.
;
vm
SPRET-spistack_add_blank.com ${refifolder}/2DRefi001 zzmyunselect blank
;
vm
SPRET-spistack_mask.com ${refifolder}/2DRefi001 ${refifolder}/2DRefi001msk ${maskfile}
;
      BP 3f
        ${refifolder}/2DRefi001msk@*******
        ${selectfile}
        ${angfile}
        *
        ${refifolder}/3DRefi001temp
;
      BP 3f
        ${refifolder}/2DRefi001msk@*******
        ${selectfile}o
        ${angfile}
        *
        ${refifolder}/tmpvol1
;
      BP 3f
        ${refifolder}/2DRefi001msk@*******
        ${selectfile}e
        ${angfile}
        *
        ${refifolder}/tmpvol2
;
[volume_end]=3*[KD]
;
[volume_end_voxels3]=[volume_end]*1000/0.81/[apix]/[apix]/[apix]
[volume_end_voxels3a]=[volume_end_voxels3]*4/5
;
[equal_protein_size]=(2*[KD]*1000/0.81)**(1/3)
[size_diff]=[size]-[equal_protein_size]
;[expansion_times]=int([size_diff]/8)
[expansion_times]=int([size]/2/8)
[tri_equal_protein_size]=[equal_protein_size]/3/[apix]
[tri_box_size]=[size]/3
;
; lp_min is in the unit of A, not voxels.
;
[lp_min]=[equal_protein_size]/2
[lp_thr]=[lpCC]
if([lp_thr].lt.[lp_min]) [lp_thr]=[lp_min]
if([lp_thr].gt.[lp_max]) [lp_thr]=[lp_max]
;
;[fq_min]=[size]/2*[apix]
[fq_min]=[equal_protein_size]
vm
proc3d ${refifolder}/3DRefi001temp.dat ${refifolder}/3DRefi001templp.mrc apix=${apix} lp={*****[lp_thr]}
cp from ccp4
${refifolder}/3DRefi001templp.mrc
${refifolder}/3DRefi001templp
n
;
fv [thr_cen]
${refifolder}/3DRefi001templp
[volume_end_voxels3a]
;
ar
${refifolder}/3DRefi001templp
${refifolder}/3DRefi001templp2
P1/[thr_cen]
;
[equal_protein_size]=(2*[KD]*1000/0.81)**(1/3)
[tri_equal_protein_size]=[equal_protein_size]/3/[apix]
;
vm
proc3d ${refifolder}/3DRefi001templp2.dat ${refifolder}/3DRefi001templp2.mrc apix=${apix} automask2={****[tri_box_size]},1,10
;
cp from ccp4
mask.mrc
mask
n
;
mu
${refifolder}/3DRefi001templp
mask
${refifolder}/3DRefi001templpmask
*
;
fs [zzd1],[zzd2],[zzd3],[maskSTD]
mask
;
if([maskSTD].eq.0) then
[cx]=0
[cy]=0
[cz]=0
endif
;
if([maskSTD].gt.0) then
cg 3 [cx],[cy],[cz]
${refifolder}/3DRefi001templpmask
[thr_cen]
endif
;
sh f
${refifolder}/3DRefi001temp
${refifolder}/3DRefi001
[zero],[zero],[zero]
;-[cx],-[cy],-[cz]
;
cp
${refifolder}/3DRefi001
${refifolder}/3Ds{***[Itotal]}
;
      pj 3q
        ${refifolder}/3DRefi001
        [hsize]
        [first]-[last]
        ${angfile}
        ${refifolder}/prjtemp@*******
;
      rf 3 [fsc_pix] [fsc_frq]
        ${refifolder}/tmpvol1
        ${refifolder}/tmpvol2
        (1)
        (0.6,1.2)
        W
        [maxang]
        (3.0)
        ${output}-rf3doc
;
;
      rf 3 [fsc_pix] [fsc_frq]
        ${refifolder}/tmpvol1
        ${refifolder}/tmpvol2
        (1)
        (0.6,1.2)
        W
        [maxang]
        (3.0)
        ${refifolder}/rf3doc{***[Itotal]}
;
;
      [lpCC]=1/[fsc_frq]*[apix]*1.5
      [lpCCcut]=[lpCC]/2
      [real_resolution]=1/[fsc_frq]*[apix]
;
;
      sd [Itotal],[fsc_pix],[fsc_frq],[real_resolution],[volume_next],[lp_count],[stable_count],[lp1],[lp2]
        ${output}-record-fsc
;
      vm
SPRET-doc_shift_statistics_refi.com 1 zz-templast-2 ${refifolder}/shifts-total{***[Itotal]}
;
      ud s,[one],[absPerSet],[absPerImg]
        zz-templast-2
;
      sd [Itotal],[absPerSet],[absPerImg],[volume_next],[lp_count],[stable_count]
        ${output}-record-shift-statistics
;
; To check the creteria: shift changes
;
        [last_shift4]=[Itotal]-0
        [last_shift3]=[Itotal]-1
        [last_shift2]=[Itotal]-2
        [last_shift1]=[Itotal]-3
;
        if([last_shift4].lt.0) [last_shift4]=0
        if([last_shift3].lt.0) [last_shift3]=0
        if([last_shift2].lt.0) [last_shift2]=0
        if([last_shift1].lt.0) [last_shift1]=0
vm
SPRET-doc_shift_statistics_refi.com 4 zz-temp-shift_sta1 ${refifolder}/shifts-total{***[last_shift1]} ${refifolder}/shifts-total{***[last_shift2]} ${refifolder}/shifts-total{***[last_shift3]} ${refifolder}/shifts-total{***[last_shift4]}
;
        vm
SPRET-doc_shift_statistics_refi.com 1 zz-temp-shift_sta2 ${refifolder}/shifts-total{***[last_shift4]}
        vm
SPRET-doc_shift_statistics_refi.com 1 zz-temp-shift_sta3 ${refifolder}/shifts{***[last_shift4]}
;
vm
cp zz-temp-shift_sta1.dat zz-temp-shift_sta11.dat
vm
cp zz-temp-shift_sta2.dat zz-temp-shift_sta21.dat
vm
cp zz-temp-shift_sta3.dat zz-temp-shift_sta31.dat
;
if([stable_count].ge.4) then
        ud s,[one],[absPerSet1],[absPerImg1]
          zz-temp-shift_sta11
        ud s,[one],[absPerSet2],[absPerImg2]
          zz-temp-shift_sta21
        ud s,[one],[absPerSet3],[absPerImg3]
          zz-temp-shift_sta31
;
if([absPerImg2].gt.0) [absChange]=abs(([absPerImg1]-[absPerImg2])/[absPerImg2])
if([absPerImg2].eq.0) [absChange]=0
;
sd [Itotal],[absPerSet1],[absPerImg1],[absPerSet2],[absPerImg2],[absChange]
${output}-record-shift-change1
sd [Itotal],[absPerImg3],[volume_next],[lp_count],[stable_count]
${output}-record-shift-change2
; 
        [judge1]=0
        if([absPerImg3].lt.0.01) [judge1]=1
        [judge2]=0
        if([absChange].le.0.01) [judge2]=1
        [judge3]=[judge1]*[judge2]
        if([judge3].eq.1) [ifstable]=1
;
;        if([absPerImg3].lt.0.01) then
;           if([absChange].le.0.01) then
;              [ifstable]=1
;           endif
;        endif
endif
;
LB91
;
    lb03
; 
    [abs_lpstop]=1
    if([Ilb02].ge.2) then
    [Itotal]
vm
cp ${output}-record-fsc.dat ${output}-record-fsctemp.dat
    ud s,[Itotal],[fp1],[fq1],[rr1]
      ${output}-record-fsctemp
[abs_lpstop]=abs([rr1]-[real_resolution_0])/[rr1]
    endif
if([abs_lpstop].le.0.01) [if_lpstop]=1
LB92
  lb02
;
lb01
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;First round, no 3D mask, no 2D mask. End
;
;
cp
${refifolder}/3Ds{***[Itotal]}
${output}-refi
;
cp
${refifolder}/2Ds{***[Itotal]}@
${output}-refi-shift@
;

vm
SPRET-projection.com ${refifolder}/3Ds000 ${output}-prj ${angfile} zz-sel-all
;
vm
SPRET-projection.com ${refifolder}/3Ds{***[Itotal]} ${output}-refi-prj ${angfile} zz-sel-all
;
en d
;
eot
#
#

proc3d ${output}-refi.dat ${output}-refi-lp008.mrc apix=${apix} lp=8
proc3d ${output}-refi.dat ${output}-refi-lp016.mrc apix=${apix} lp=16
proc3d ${output}-refi.dat ${output}-refi-lp026.mrc apix=${apix} lp=26
# rm -r ${refifolder}
rm ${output}-mixture.dat
#
#SPRET-spistack_mixture.com [total_nums_of_input_stacks] [number_of_images_in_each_stack] [output_no_ext] [input1_no_ext] [input2_no_ext] [input3_no_ext] [input4_no_ext] ... ...
SPRET-spistack_mixture.com 6 ${totalNum} ${output}-mixture ${input} ${output}-prj ${output}-refi-shift ${refifolder}/2DRefi001lp ${output}-refi-prj ${refifolder}/prjtemplp

rm LOG.dat
rm ${output}tempo.dat
rm ${output}tempe.dat
rm blank.dat
rm zztemplp.dat zztemplp.mrc
