#!/bin/csh
#
#
#   Generate angle files for tomography, which means the files only contain theta angle, and phi, psi are both zero.
#
#   usage: SPRET-create_angle.com [output_no_ext] [start_angle] [end_angle] [angle_step/angle_numbers] [index:0 for angle_step,1 for angle_numbers]
#
#      where:
#
#          [output_no_ext]:            output angle file in Spider doc format, without extension .dat.
#
#          [start_angle]:              starting angle.
#
#          [end_angle]:                end angle.
#
#          [angle_step/angle_numbers]: angle tilt step (positive or negative), or total angle numbers.
#
#          [index]:                    0 represents the above input number is angle_step,1 represents the above input number is total angle_numbers.
#
#
#
#   Users have two choices to generate the angle files, one is to define the angle step, another is to define the total angle numbers within the range limited by [start_angle] and [end_angle].
#
#	Lei Zhang @ REN Lab, UCSF, Wed Feb  4 11:41:11 PST 2009
#
#
#-----------------------------------------------------------------------------------------------------------------------------------------------
#
if ($#argv != 5) then
  echo "Generate angle files for tomography."
  echo "usage: SPRET-create_angle.com [output_no_ext] [start_angle] [end_angle] [angle_step/angle_numbers] [index:0 for angle_step,1 for angle_numbers]"
  echo "such as: SPRET-create_angle.com angcal -70.5 70.5 1.5 0"
  exit
endif
#
#
set output=$1
set startang=$2
set endang=$3
set angSN=$4
set index=$5
#
#
spider dat << eot
[start]=${startang}
[end]=${endang}
[stmp]=${angSN}
[step]=abs([stmp])
[index]=${index}
;
[mm]=[end]-[start]
if([mm].gt.0) [direction]=1
if([mm].lt.0) [direction]=-1
;
if([index].eq.0) then
  [nums]=int(abs([end]-[start])/[step])+1
;
  de
    ${output}
;
do lb1 [xlb1]=1,[nums]
; 
    [psi]=0
    [phi]=0
    [theta]=[start]+[direction]*([xlb1]-1)*[step]
;
    sd [xlb1],[psi],[theta],[phi]
    ${output}
;
lb1
;
sd e
${output}
endif
;
if([index].eq.1) then
;
  [nums]=[step]
  [delta]=abs(([end]-[start])/([nums]-1))
;
  de
    ${output}
;
  do lb2 [xlb2]=1,[nums]
;
    [psi]=0
    [phi]=0
    [theta]=[start]+[direction]*([xlb2]-1)*[delta]
;
    sd [xlb2],[psi],[theta],[phi]
      ${output}
;
  lb2
;
sd e
${output}
;
endif
;
en d
eot
rm LOG.dat
