#!/bin/csh
#
#
#   Do the projection based on the input Spider volume file, angle file, and select file.
#
#   usage: SPRET-projection.com [input_vol_no_ext] [output_prefix] [angle_file_noext] [select_file_noext]
#
#      where:
#
#          [input_vol_no_ext] :  the input Spider volume file, without extension .dat.
#
#          [output-prefix]    :  the prefix of the output projections.
#
#          [angle_file_noext] :  the file contains Euler Angles for the projection, could be created by SPRET-create_angle.com, and without extension .dat.
#
#          [select_file_noext]:  the file contains particle selection for the projection, could be created by SPRET-create_select.com, and without extension .dat.
#
#
#          
#	Lei Zhang @ REN Lab, UCSF, Thu Feb  5 16:27:39 PST 2009
#
#   Revise [hsize] from int([size]/2)-2 to int([size]/2), Feb 20 2009 Lei
#
#
#-----------------------------------------------------------------------------------
if ($#argv != 4) then
  echo "Do the projection based on the input Spider volume file, angle file, and select file."
  echo "usage: SPRET-projection.com [input_vol_no_ext] [output_prefix] [angle_file_noext] [select_file_noext]"
  echo "such as: SPRET-projection.com vol prj angcal myselect"
  exit
endif
#
set input=$1
set output=$2
set angfile=$3
set selfile=$4
#
rm ${output}.dat
spider dat << eot
;
fi [size]
  ${input}
  2
;
[hsize]=int([size]/2)
;
pj 3q
  ${input}
  [hsize]
  ${selfile}
  ${angfile}
  ${output}@*******
en d
eot
rm LOG.dat
