#!/bin/bash
#
#
#   Rendering 2D images by using the algorithom of EMAN (boxer or v2).
#
#   usage: SPRET-spistack_renderEman.com [input_dat_no_ext] [output_dat_no_ext] [brightness] [contrast] [scale_pixel_to_1to254_or_not]
#
#      where:
#
#          [input_dat_no_ext]:  the input Spider stack file's name without extension.
#
#          [output_no_ext]: output Spider stack file's name without extension.
#
#          [brightness]:  the brightness of image according to eman v2 or boxer.
#
#          [output_no_ext]: the contrast of image according to eman v2 or boxer.
#
#          [Bool_scale_pix_to_1to254]: whether scale pixel valve to 1 to 254 as EMAN does.
#
#
#   Dongsheng Lei @ REN Lab, LBNL, Mon Oct 10 2016
#
#
#
#---------------------------------------------------------------------------------
if [[ $# -ne 5 ]]; then
  echo "usage: SPRET-spistack_renderEman.com [input_dat_no_ext] [output_dat_no_ext] [brightness] [contrast] [scale_pixel_to_1to254_or_not]"
  echo "such as: SPRET-spistack_renderEman.com stack1 stack1_rend 0.1 0.2 0"
  exit
fi
#
input=$1
output=$2
rm $output.dat
nums=`iminfo ${input}.dat | grep contains | awk '{print $3}'`
spider dat << EOF
[first]=1
[last]=${nums}
[nums]=[last]-[first]+1
[brightness]=$3
[contrast]=$4
[scale_bool]=$5
[maxCont]=1.001
[minScale]=1.0
[maxScale]=254.0
cp
${input}@
_1@
[nums]
cp 
_1@
_2@
[nums]
if([brightness].GT.0) then
	if([scale_bool].EQ.0) then 
		do lb1 [x1]=1,[nums]
			fs [max],[min],[avg],[std]
			_1@{*******[x1]}
			[mm]=[max]-[avg]
			[mmalt]=[avg]-[min]
			if([mmalt].GT.[mm]) then
				[mm]=[mmalt]
			endif
			[b]=[avg]-[brightness]*([avg]-[min])
			[wf]=(1.001-[contrast])*([maxCont]-[contrast])*[mm]
			[lowcut]=[b]-[wf]
			[upcut]=[b]+[wf]
			th
			_1@{*******[x1]}
			_2@{*******[x1]}
			C
			[upcut],[lowcut]
		lb1
		cp
		_2@
		${output}@
	endif
        if([scale_bool].GT.0) then
		do lb1 [x1]=1,[nums]
			fs [max],[min],[avg],[std]
			_1@{*******[x1]}
			[mm]=[max]-[avg]
			[mmalt]=[avg]-[min]
			if([mmalt].GT.[mm]) then
				[mm]=[mmalt]
			endif
			[b]=[avg]-[brightness]*([avg]-[min])
			[wf]=(1.001-[contrast])*([maxCont]-[contrast])*[mm]
			[lowcut]=[b]-[wf]
			[upcut]=[b]+[wf]
			th
			_1@{*******[x1]}
			_2@{*******[x1]}
			C
			[upcut],[lowcut]
			ar sca
			_2@{*******[x1]}
			_1@{*******[x1]}
			[minScale] [maxScale]
		lb1
		cp
		_1@
		${output}@
	endif
endif
if([brightness].LE.0) then
	if([scale_bool].EQ.0) then 
		do lb1 [x1]=1,[nums]
			fs [max],[min],[avg],[std]
			_1@{*******[x1]}
			[mm]=[max]-[avg]
			[mmalt]=[avg]-[min]
			if([mmalt].GT.[mm]) then
				[mm]=[mmalt]
			endif
			[b]=[avg]-[brightness]*([max]-[avg])
			[wf]=([maxCont]-[contrast])*([maxCont]-[contrast])*[mm]
			[lowcut]=[b]-[wf]
			[upcut]=[b]+[wf]
			th
			_1@{*******[x1]}
			_2@{*******[x1]}
			C
			[upcut],[lowcut]
		lb1
		cp
		_2@
		${output}@
	endif
        if([scale_bool].GT.0) then
		do lb1 [x1]=1,[nums]
			fs [max],[min],[avg],[std]
			_1@{*******[x1]}
			[mm]=[max]-[avg]
			[mmalt]=[avg]-[min]
			if([mmalt].GT.[mm]) then
				[mm]=[mmalt]
			endif
			[b]=[avg]-[brightness]*([max]-[avg])
			[wf]=(1.001-[contrast])*([maxCont]-[contrast])*[mm]
			[lowcut]=[b]-[wf]
			[upcut]=[b]+[wf]
			th
			_1@{*******[x1]}
			_2@{*******[x1]}
			C
			[upcut],[lowcut]
			ar sca
			_2@{*******[x1]}
			_1@{*******[x1]}
			[minScale] [maxScale]
		lb1
		cp
		_1@
		${output}@
	endif
endif
en d
EOF
