#!/usr/bin/python
from numpy import *
from math import sqrt
from writeMRCfile import *
from readMRCfile import *
from os import system
from os import remove
import os
import sys

#   Do the Missing Wedge correct. 
#
#   1. Shift original point to the center in FT space, ver1.1. 
#   2. Add small cylinder part in center of realwedge mask, ver1.2. 
#   3. Use cutoff lp 3D data replace raw data (lp fix to 4A), ver1.3. 
#
#
#   "usage: ./runMissingWedge.py [input_3Dmap_mrc_no_ext] [3Dmask_mrc_no_ext] [negative_angle] [positive_angle] [iteration_number]
#
#      where:
#
#          [input_3Dmap_mrc]:    the input MRC 3D density map file to do Missing Wedge correction.
#     
#          [3Dmask_mrc]:         the input MRC 3D mask density map, cutoff mask is better.
#
#          [negative_angle]:     the bottom negative angle for 2D stack.
#
#          [positive_angle]:     the top positive angle for 2D stack.
#
#          [iteration_number]:   the number of iterations.
#
#    output name is "[input_3Dmap_mrc_no_ext]-MWcor.mrc"
#
#
#	Xiaobo Zhai @ REN Lab, LBNL, Fri Oct  7 16:54:38 PDT 2016
#
if len(sys.argv) != 6:
    print "Please enter 5 arguments: volume_mrc_no_ext, 3Dmask_mrc_no_ext, negative angle, positive angle, iteration num"
    print "Example: ./runMissingWedge.py vol_BP_01 3DMask006-adv -70 70 500"
    print "Argument list " + str(sys.argv)
    exit(1)
else:
    input_v = sys.argv[1]
    mask = sys.argv[2]
    n_ang  = int(sys.argv[3])
    p_ang  = int(sys.argv[4])    
    numIterations = 1 + int(sys.argv[5])
 #   Apix=float(sys.argv[6])

volume = input_v + ".mrc"
mask = mask + ".mrc"

(vol_BP, b, c) = readMRCfile(volume)
size = len(vol_BP)

## creat wedge_mask
# For the matrix in python and chimera, x and y axies are exchanged.
# In Python, number of array is from 0. So mid_point is real original point -1.
mid_point = int(round((float(size)-1)/2))
RealWedge = zeros((size,size,size))
one_matrix = ones((size,size,size))

# range(a) is from 0 to a-1
for x in range(size):
    for z in range(size):
        for y in range(size):
            if z == mid_point:
                  angle = 90
            else:
            	  angle = degrees(arctan(float(x - mid_point)/float(z - mid_point)))
            if (angle <= p_ang and angle >= n_ang):
                  RealWedge[x][y][z] = 1
            RealWedge[mid_point][y][mid_point] = 1

# add small cylinder mask in center of realwedge. range(a,b) is from a to b-1
for x in range(mid_point - 1, mid_point + 2):
    for z in range(mid_point - 1, mid_point + 2):   
	     for y in range(size):
                  RealWedge[x][y][z] = 1
                                         
MissWedge = one_matrix - RealWedge

writeMRCfile(RealWedge,b,c,"mask_realwedge.mrc",'float') 
writeMRCfile(MissWedge,b,c,"mask_misswedge.mrc",'float') 

del RealWedge
del MissWedge
##
##
# creat cutoff lp mask
lpv = 3
Mask_lp = zeros((size,size,size))
#crit = Apix / lpv * size
crit = size / lpv
crit_r = int(crit) + 1
if (crit_r < size/2):
    x1 = mid_point - crit_r
    x2 = mid_point + crit_r
else:
    x1 = 0
    x2 = size - 1

for x in range(x1, x2 + 1):
    for z in range(x1, x2 + 1):
        for y in range(x1, x2 + 1):
            lpr2 = (x-mid_point)**2+(y-mid_point)**2+(z-mid_point)**2
            if (lpr2 <= crit_r**2):
                    Mask_lp[x][y][z] = 1

writeMRCfile(Mask_lp,b,c,"mask_lp.mrc",'float') 
del Mask_lp
##

##
# Missing Wedge correct for lp_raw3D
del vol_BP
(vol_BP, b, c) = readMRCfile(volume)
(Mask_lp , b, c) = readMRCfile("mask_lp.mrc")
vol_BP_FT = fft.fftn(vol_BP)
vol_BP_FTs = fft.fftshift(vol_BP_FT)
vol_BP_FTs_lp = vol_BP_FTs * Mask_lp 

vol_BP_FT_lp = fft.ifftshift(vol_BP_FTs_lp)
vol_BP_lp = fft.ifftn(vol_BP_FT_lp)
vol_BP_lp = real(vol_BP_lp)

(wedge, b, c) = readMRCfile("mask_realwedge.mrc")
FT_unchanged = wedge * vol_BP_FTs_lp

for ii in range(1, numIterations):
    if (ii == 1 or (ii % 50) == 0):
        print "Starting iteration", ii

    (mask_real, b, c) = readMRCfile(mask)
    vol_masked = vol_BP_lp * mask_real
    del vol_BP_lp
    del mask_real
    vol_masked = vol_masked - (vol_masked * (vol_masked < 0))

    vol_masked_FT = fft.fftn(vol_masked)
    vol_masked_FTs = fft.fftshift(vol_masked_FT)

    del vol_masked

    (wedge_rev, b, c) = readMRCfile("mask_misswedge.mrc")
    vol_masked_FT_wedge_rev = vol_masked_FTs * wedge_rev

    del vol_masked_FT
    del vol_masked_FTs
    del wedge_rev

    vol_masked_FT_news = vol_masked_FT_wedge_rev + FT_unchanged
    del vol_masked_FT_wedge_rev

    vol_masked_FT_new = fft.ifftshift(vol_masked_FT_news) 
    vol_BP_lp = fft.ifftn(vol_masked_FT_new)

    del vol_masked_FT_new
    del vol_masked_FT_news

    vol_BP_lp = real(vol_BP_lp)

filenameIter = input_v + "-MWcor.mrc"
writeMRCfile(vol_BP_lp, b, c, filenameIter, "float")
